unit FavoritesTreeView;

//==============================================================================
//21 Dec, 1999 : Added AutoLoad property, and HasItems property
//          By : MrPMorris@Hotmail.com  (aka cubud)
//==============================================================================

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Favorites, Shdocvw;

type
  TFavoritesTreeView = class(TCustomTreeView)
  private
    { Private declarations }
    FAutoLoad : Boolean;
    FFavoritesTree : Pointer;
    FImageList : TImageList;
    FOnMouseEnter,
    FOnMouseLeave : TNotifyEvent;
    FOnNavigate : TFavoritesNavigateEvent;

    procedure CMMouseEnter(var Message : TMessage); message CM_MouseEnter;
    procedure CMMouseLeave(var Message : TMessage); message CM_MouseLeave;
    function GetAddToFavoritesCaption: String;
    function GetExportFavoritesCaption: String;
    function GetImportFavoritesCaption: String;
    function GetOptions: TOptions;
    function GetOrganizeFavoritesCaption: String;
    function GetWebbrowser: TWebbrowser;
    procedure SetAddToFavoritesCaption(const Value: String);
    procedure SetExportFavoritesCaption(const Value: String);
    procedure SetImportFavoritesCaption(const Value: String);
    procedure SetOptions(const Value: TOptions);
    procedure SetOrganizeFavoritesCaption(const Value: String);
    procedure SetWebbrowser(const Value: TWebbrowser);
    function GetCaption: String;
    procedure SetCaption(const Value: String);
  protected
    { Protected declarations }
    procedure Click; override;
    procedure Delete(Node: TTreeNode); override;
    procedure DoMouseEnter;
    procedure DoMouseLeave;
    procedure DoNavigateEvent(Sender : TObject; Url : WideString; var Allow : Boolean); virtual;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure CreateMenu; virtual;
    function  HasItems : Boolean;
    procedure UpdateMenu; virtual;
  published
    { Published declarations }
    property AddToFavoritesCaption  : String
      read GetAddToFavoritesCaption
      write SetAddToFavoritesCaption;
    property AutoLoad               : boolean
      read FAutoLoad
      write FAutoLoad;
    property Caption                : String
      read GetCaption
      write SetCaption;
    property ExportFavoritesCaption : String
      read GetExportFavoritesCaption
      write SetExportFavoritesCaption;
    property ImportFavoritesCaption : String
      read GetImportFavoritesCaption
      write SetImportFavoritesCaption;
    property OrganizeFavoritesCaption : String
      read GetOrganizeFavoritesCaption
      write SetOrganizeFavoritesCaption;
    property Options: TOptions
      read GetOptions
      write SetOptions;
    property Webbrowser: TWebbrowser
      read GetWebbrowser
      write SetWebbrowser;

    property OnMouseEnter : TNotifyEvent
      read FOnMouseEnter
      write FOnMouseEnter;
    property OnMouseLeave : TNotifyEvent
      read FOnMouseLeave
      write FOnMouseLeave;
    property OnNavigate   : TFavoritesNavigateEvent
      read FOnNavigate
      write FOnNavigate;

    //inherited stuff
    property Align;
    property Anchors;
    property AutoExpand;
    property BidiMode;
    property BorderStyle;
    property BorderWidth;
    property ChangeDelay;
    property Color;
    property Constraints;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property HotTrack;
    property Indent;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ParentCtl3D;
    property PopupMenu;
    property RightClickSelect;
    property RowSelect;
    property ShowButtons;
    property ShowHint;
    property ShowLines;
    property ShowRoot;
    property TabOrder;
    property TabStop;
    property ToolTips;
    property Visible;

    property OnAdvancedCustomDraw;
    property OnAdvancedCustomDrawItem;
    property OnChange;
    property OnChanging;
    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnContextPopup;
    property OnCustomDraw;
    property OnCustomDrawItem;
    property OnDblClick;
    property OnDeletion;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnExpanding;
    property OnExpanded;
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Internet', [TFavoritesTreeView]);
end;

type
  TTreeData = record
    Caption                   : String[255];
    URL                       : String[255];
    OnClick                   : TNotifyEvent;
  end;
  PTreeData = ^TTreeData;

  TFavoritesTree = class(TAbstractFavorites)
  private
    FCaption : String;
    FTreeView : TTreeView;
  protected
    procedure ClearItems; override;
    function CreateItem(const ACaption, aURL: string; AEnabled: Boolean;
                     AOnClick: TNotifyEvent): TObject; override;
    function CreateMainItem(AEnabled: Boolean; AOnClick: TNotifyEvent): TObject; Override;
    function GetURL(aItem : TObject) : String; Override;
    function InsertFolder(aParent : TObject; Const Folder, Caption : String) : TObject; override;
    function InsertItem(aParent, aChild : TObject; aIndex : Integer) : TObject; override;
    function ParentCount(aParent : TObject) : Integer; override;
    procedure SetIcon(aItem : TObject; aIcon : TIcon); override;
  public
    constructor Create(AOwner : TComponent); override;

    procedure CreateMenu; override;
    procedure UpdateMenu; override;
  published
    property Caption : String read FCaption write FCaption;
    property TreeView : TTreeView read FTreeView write FTreeView;
  end;


{ TFavoritesTree }

procedure TFavoritesTree.ClearItems;
begin
  TreeView.Items.Clear;
  if TreeView.Images <> nil then
    TreeView.Images.Clear;
end;

constructor TFavoritesTree.Create(AOwner: TComponent);
begin
  inherited;
  FCaption := 'Favorites';
  AddToFavoritesCaption := 'Add to Favorites';
  OrganizeFavoritesCaption := 'Organize Favorites';
  ImportFavoritesCaption := 'Import Favorites';
  ExportFavoritesCaption := 'Export Favorites';
end;

function TFavoritesTree.CreateItem(const ACaption, aURL: string;
  AEnabled: Boolean; AOnClick: TNotifyEvent): TObject;
var
  NewData                     : PTreeData;
begin
  Result := TreeView.Items.AddChild(nil, ACaption);
  with TTreeNode(Result) do begin
    Getmem(NewData, SizeOf(TTreeData));
    NewData.Caption := aCaption;
    NewData.OnClick := aOnClick;
    NewData.URL := aURL;
    Data := NewData;
  end;
end;

function TFavoritesTree.CreateMainItem(AEnabled: Boolean;
  AOnClick: TNotifyEvent): TObject;
begin
  Result := TreeView.items.AddFirst(nil,FCaption);
end;

procedure TFavoritesTree.CreateMenu;
begin
  TreeView.Items.BeginUpdate;
  ClearItems;
  inherited;
  TreeView.Items.EndUpdate;
end;

function TFavoritesTree.GetURL(aItem: TObject): String;
begin
  with TTreeNode(aItem) do begin
    if Data <> nil then
      Result := PTreeData(Data).URL;
  end;
end;

function TFavoritesTree.InsertFolder(aParent: TObject; const Folder,
  Caption: String): TObject;
var
  TheData                     : PTreeData;
begin
  Result := TreeView.items.AddChild(TTreeNode(aParent), Caption);
  with TTreeNode(Result) do begin
    Text := Caption;
    GetMem(TheData, SizeOf(TTreeData));
    TheData.Caption := Caption;
    TheData.OnClick := nil;
    Data := TheData;
  end;
end;

function TFavoritesTree.InsertItem(aParent, aChild: TObject;
  aIndex: Integer): TObject;
begin
  TTreeNode(aChild).MoveTo(TTreeNode(aParent), naAddChild);
end;

function TFavoritesTree.ParentCount(aParent: TObject): Integer;
begin
  Result := 1;
end;

procedure TFavoritesTree.SetIcon(aItem: TObject; aIcon: TIcon);
var
  Icon : TIcon;
  NewIndex : Integer;
begin
  if TreeView.Images = nil then exit;
  with TreeView.Images do begin
    Icon := TIcon.Create;
    Icon.Assign(aIcon);
    NewIndex := AddIcon(Icon);
    with TTreeNode(aItem) do begin
      ImageIndex := NewIndex;
      SelectedIndex := NewIndex;
      StateIndex := NewIndex;
    end;
  end;

end;

procedure TFavoritesTree.UpdateMenu;
var
  Save_Cursor: TCursor;
begin
  Save_Cursor := Screen.Cursor;
  Screen.Cursor := crHourglass;
  CreateMenu;
  Screen.Cursor := Save_Cursor;
end;



{ TFavoritesTreeView }

procedure TFavoritesTreeView.Click;
begin
  inherited;
  if Selected = nil then exit;
  if Selected.Data <> nil then
    if Assigned(PTreeData(Selected.Data).OnClick) then
      PTreeData(Selected.Data).OnClick(Selected);
end;

procedure TFavoritesTreeView.CMMouseEnter(var Message: TMessage);
begin
  DoMouseEnter;
end;

procedure TFavoritesTreeView.CMMouseLeave(var Message: TMessage);
begin
  DoMouseLeave;
end;

constructor TFavoritesTreeView.Create(AOwner: TComponent);
begin
  inherited;
  FFavoritesTree := TFavoritesTree.Create(Self);
  TFavoritesTree(FFavoritesTree).OnNavigate := DoNavigateEvent;
  FImageList := TImageList.Create(Self);
  Images := FImageList;
  TFavoritesTree(FFavoritesTree).TreeView := TTreeView(Self);
  TFavoritesTree(FFavoritesTree).Options := [AddFavorites, OrganizeFavorites, ImportFavorites, ExportFavorites];
  ReadOnly := True;
  FAutoLoad := True;
end;

procedure TFavoritesTreeView.CreateMenu;
begin
  TFavoritesTree(FFavoritesTree).CreateMenu;
end;

procedure TFavoritesTreeView.Delete(Node: TTreeNode);
begin
  if Node.data <> nil then FreeMem(Node.Data);
  inherited;
end;

destructor TFavoritesTreeView.Destroy;
begin
  FImageList.Free;
  TFavoritesTree(FFavoritesTree).Free;
  inherited;
end;

procedure TFavoritesTreeView.DoMouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TFavoritesTreeView.DoMouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TFavoritesTreeView.DoNavigateEvent(Sender: TObject;
  Url: WideString; var Allow: Boolean);
begin
  if Assigned(FOnNavigate) then FOnNavigate(Self, URL, Allow);
end;

function TFavoritesTreeView.GetAddToFavoritesCaption: String;
begin
  Result := TFavoritesTree(FFavoritesTree).AddToFavoritesCaption;
end;

function TFavoritesTreeView.GetCaption: String;
begin
  Result := TFavoritesTree(FFavoritesTree).Caption;
end;

function TFavoritesTreeView.GetExportFavoritesCaption: String;
begin
  Result := TFavoritesTree(FFavoritesTree).ExportFavoritesCaption;
end;

function TFavoritesTreeView.GetImportFavoritesCaption: String;
begin
  Result := TFavoritesTree(FFavoritesTree).ImportFavoritesCaption;
end;

function TFavoritesTreeView.GetOptions: TOptions;
begin
  Result := TFavoritesTree(FFavoritesTree).Options;
end;

function TFavoritesTreeView.GetOrganizeFavoritesCaption: String;
begin
  Result := TFavoritesTree(FFavoritesTree).OrganizeFavoritesCaption;
end;

function TFavoritesTreeView.GetWebbrowser: TWebbrowser;
begin
  Result := TFavoritesTree(FFavoritesTree).Webbrowser;
end;

function TFavoritesTreeView.HasItems: Boolean;
begin
  Result := Items.Count > 0;
end;

procedure TFavoritesTreeView.KeyPress(var Key: Char);
begin
  inherited;
  if Key=#13 then Click;
end;

procedure TFavoritesTreeView.Loaded;
begin
  inherited;
  if (WebBrowser <> nil) and (AutoLoad) then CreateMenu;
end;

procedure TFavoritesTreeView.SetAddToFavoritesCaption(const Value: String);
begin
  TFavoritesTree(FFavoritesTree).AddToFavoritesCaption := Value;
end;

procedure TFavoritesTreeView.SetCaption(const Value: String);
begin
  TFavoritesTree(FFavoritesTree).Caption := Value;
end;

procedure TFavoritesTreeView.SetExportFavoritesCaption(
  const Value: String);
begin
  TFavoritesTree(FFavoritesTree).ExportFavoritesCaption := Value;
end;

procedure TFavoritesTreeView.SetImportFavoritesCaption(
  const Value: String);
begin
  TFavoritesTree(FFavoritesTree).ImportFavoritesCaption := Value;
end;

procedure TFavoritesTreeView.SetOptions(const Value: TOptions);
begin
  TFavoritesTree(FFavoritesTree).Options := Value;
end;

procedure TFavoritesTreeView.SetOrganizeFavoritesCaption(
  const Value: String);
begin
  TFavoritesTree(FFavoritesTree).OrganizeFavoritesCaption := Value;
end;

procedure TFavoritesTreeView.SetWebbrowser(const Value: TWebbrowser);
begin
  TFavoritesTree(FFavoritesTree).Webbrowser := Value;
end;

procedure TFavoritesTreeView.UpdateMenu;
begin
  TFavoritesTree(FFavoritesTree).UpdateMenu;
end;

end.
