
//***************************************
//       IEParser ver 1.00              *
//        For Delphi 4 og 5             *
//       Freeware Component             *
//               by                     *
//                                      *
//        Per Linds Larsen             *
//                                      *
//   http://www.euromind.com/iedelphi   *
//***************************************

unit IEParser;

interface

uses
{$IFDEF VER130}mshtml{$ELSE}mshtml_tlb{$ENDIF},
  Dialogs, Sysutils, Urlmon, ActiveX, Windows, Messages, Classes;

const
  WM_USER_STARTWALKING = WM_USER + 1;
  DISPID_AMBIENT_DLCONTROL = (-5512);
  READYSTATE_COMPLETE = $00000004;

  DLCTL_DLIMAGES = $00000010;
  DLCTL_VIDEOS = $00000020;
  DLCTL_BGSOUNDS = $00000040;
  DLCTL_NO_SCRIPTS = $00000080;
  DLCTL_NO_JAVA = $00000100;
  DLCTL_NO_RUNACTIVEXCTLS = $00000200;
  DLCTL_NO_DLACTIVEXCTLS = $00000400;
  DLCTL_DOWNLOADONLY = $00000800;
  DLCTL_NO_FRAMEDOWNLOAD = $00001000;
  DLCTL_RESYNCHRONIZE = $00002000;
  DLCTL_PRAGMA_NO_CACHE = $00004000;
  DLCTL_FORCEOFFLINE = $10000000;
  DLCTL_NO_CLIENTPULL = $20000000;
  DLCTL_SILENT = $40000000;
  DLCTL_OFFLINEIFNOTCONNECTED = $80000000;
  DLCTL_OFFLINE = DLCTL_OFFLINEIFNOTCONNECTED;

type

  TElementInfo = record
    className: string;
    id: string;
    tagName: string;
    title: string;
    language: string;
    sourceIndex: Integer;
    recordNumber: OleVariant;
    lang: string;
    offsetLeft: Integer;
    offsetTop: Integer;
    offsetWidth: Integer;
    offsetHeight: Integer;
    innerHTML: string;
    innerText: string;
    outerHTML: string;
    outerText: string;
  end;

  TOnDownloadCompleteEvent = procedure(Sender: TObject) of object;
  TOnBREvent = procedure(Sender: TObject; clear: string; Element: TElementInfo) of object;
  TOnHREvent = procedure(Sender: TObject; align: string; color, width, size: OleVariant;
    noShade: Boolean; Element: TElementInfo) of object;
  TOnDIVEvent = procedure(Sender: TObject; align: string; noWrap: Boolean; Element: TElementInfo) of object;
  TOnScriptEvent = procedure(Sender: Tobject; src, htmlFor, event, text: string; defer: Boolean; Element: TElementInfo) of object;
  TOnFormEvent = procedure(Sender: TObject; action, dir, encoding, method,
    target, name: string; Element: TElementInfo) of object;
  TOnMarqueeEvent = procedure(Sender: TObject; bgcolor, width, height: OleVariant;
    direction, behavior: string; ScrollAmount, scrollDelay, loop, vspace, hspace: Integer; Element: TElementInfo) of object;
  TOnFontEvent = procedure(Sender: TObject; color, size: OleVariant; Face: string; Element: TElementInfo) of object;
  TOnBaseFontEvent = procedure(Sender: TObject; color: OleVariant; Face: string;
    size: Integer; Element: TElementInfo) of object;
  TOnBaseEvent = procedure(Sender: TObject; href, target: string; Element: TElementInfo) of object;
  TOnMetaEvent = procedure(Sender: TObject; httpEquiv, content, name, url,
    charset: string; Element: TElementInfo) of object;
  TOnBodyEvent = procedure(Sender: TObject; background, bgProperties: string;
    leftmargin, topmargin, rightMargin, bottomMargin, bgColor, text,
    link, vlink, alink: OleVariant; noWrap: Boolean; Element: TElementInfo) of object;
  TOnImageEvent = procedure(Sender: TObject; src, lowsrc, vrml, dynsrc,
    alt, align, usemap: string; isMap: Boolean; border, loop: OleVariant;
    vspace, hspace, width, height: Integer; Element: TElementInfo) of object;
  TOnAnchorEvent = procedure(Sender: TObject; href, target, rel, rev, urn, Methods,
    name, host, hostname, pathname, port, protocol, search, hash, accesskey,
    protocolLong, mimeType, nameProp: string; Element: TElementInfo) of object;
  TOnCommentEvent = procedure(sender: TObject; text: string; Element: TElementInfo) of object;
  TOnElementEvent = procedure(Sender: TObject; ElementInfo: TElementInfo) of object;

  TIEParser = class(TComponent, IUnknown, IPropertyNotifySink, IOleClientSite)
  protected
    Element: TElementInfo;
    FAbout: String;
    FRefCount: Integer;
    FScript: TOnScriptEvent;
    FDownLoadComplete: TOnDownLoadCompleteEvent;
    FAnchor: TOnAnchorEvent;
    FComment: TOnCommentEvent;
    FElement: TOnElementEvent;
    FBody: TOnBodyEvent;
    FMeta: TOnMetaEvent;
    FBase: TOnBaseEvent;
    FBaseFont: TOnBasefontEvent;
    FFont: TOnFontEvent;
    FImage: TOnImageEvent;
    FMarquee: TOnMarqueeEvent;
    FForm: TOnFormEvent;
    FDiv: TOnDivEvent;
    FHr: TOnHrEvent;
    FBr: TOnBrEvent;
    FDownloadOnly: Boolean;
    FUrl: string;
    _URL: PwideChar;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
    function OnChanged(dispid: TDispID): HResult; stdcall;
    function OnRequestEdit(dispid: TDispID): HResult; stdcall;
    function SaveObject: HResult; stdcall;
    function GetMoniker(dwAssign: Longint; dwWhichMoniker: Longint;
      out mk: IMoniker): HResult; stdcall;
    function GetContainer(out container: IOleContainer): HResult; stdcall;
    function ShowObject: HResult; stdcall;
    function OnShowWindow(fShow: BOOL): HResult; stdcall;
    function RequestNewObjectLayout: HResult; stdcall;
    function LoadUrlFromMoniker: HResult;
  public
    Doc: IhtmlDocument2;
    All: IHtmlElementCollection;
    procedure Execute;
  published
    property About: String read FAbout write FAbout;
    property Url: string read FURL write FURL;
    property DownloadOnly: Boolean read FDownloadOnly write FDownloadOnly;
    property OnAnchor: TOnAnchorEvent read FAnchor write FAnchor;
    property OnComment: TOnCommentEvent read FComment write FComment;
    property OnElement: TOnElementEvent read FElement write FElement;
    property OnImage: TOnImageEvent read FImage write FImage;
    property OnBody: TOnBodyEvent read FBody write FBody;
    property OnMeta: TOnMetaEvent read FMeta write FMeta;
    property OnBase: TOnBaseEvent read FBase write FBase;
    property OnFont: TOnFontEvent read FFont write FFont;
    property OnMarquee: TOnMarqueeEvent read FMarquee write FMarquee;
    property OnBaseFont: TOnBaseFontEvent read FBaseFont write FBaseFont;
    property OnForm: TOnFormEvent read FForm write FForm;
    property OnScript: TOnScriptEvent read FScript write FScript;
    property OnDiv: TOnDivEvent read FDiv write FDiv;
    property OnHR: TOnHREvent read Fhr write Fhr;
    property OnBR: TOnBREvent read FBr write FBr;
    property OnDownloadComplete: TOnDownloadCompleteEvent read FDownloadComplete write FDownloadComplete;
  end;

implementation


procedure TIEParser.Execute;
var
  boolWorking: Boolean;
  E: IhtmlElement;
  X, C: Integer;
  ConnectionPoint: IConnectionPoint;
  OleChar: array[0..MAX_PATH - 1] of TOleChar;
  Msg: TMsg;
  hr: HRESULT;
begin
  CoCreateInstance(CLASS_HTMLDocument, nil, CLSCTX_INPROC_SERVER, IHTMLDocument2, Doc);
  _Url := @OleChar;
  MultibytetoWideChar(CP_ACP, 0, PChar(FUrl), -1, _Url, sizeof(OleChar));
  (Doc as IOleObject).SetClientSite(self as IOleClientsite);
  (Doc as IOleControl).OnAmbientPropertyChange(DISPID_AMBIENT_DLCONTROL);
  (Doc as IConnectionPointContainer).FindConnectionPoint(IpropertyNotifySink, ConnectionPoint);
  ConnectionPoint.Advise(Self as IPropertyNotifySink, C);
  HR := LoadUrlFromMoniker;
  boolWorking := System.True;
  if ((SUCCEEDED(HR)) or (HR = E_PENDING)) then
    while (boolWorking and GetMessage(Msg, 0, 0, 0)) do
    begin
      if ((Msg.Message = WM_USER_STARTWALKING) and (Msg.hWnd = 0)) then
      begin
        boolWorking := System.False;
        All := Doc.Get_all;
        if (All <> nil) and not FDownloadOnly then
          for x := 0 to All.length - 1 do begin
            E := All.item(x, 0) as IHTMLElement;
            with Element do begin
              className := E.classname;
              id := E.id;
              tagName := E.Tagname;
              title := E.Title;
              language := E.language;
              sourceIndex := E.SourceIndex;
              recordNumber := E.RecordNumber;
              lang := E.lang;
              offsetLeft := E.OffsetLeft;
              offsetTop := E.OffsetTop;
              offsetWidth := E.OffsetWidth;
              offsetHeight := E.offsetHeight;
              innerHTML := E.innerHTML;
              innerText := E.InnerText;
              outerHTML := E.OuterHTML;
              outerText := E.OuterText;
            end;

            if Assigned(FElement) then FElement(self, Element);

            if (E.Tagname = 'A') and Assigned(FAnchor) then
              with All.item(x, 0) as IHtmlAnchorElement do
                FAnchor(self, href, target, rel, rev, urn, Methods,
                  name, host, hostname, pathname, port, protocol, search, hash, accesskey,
                  protocolLong, mimeType, nameProp, Element) else

              if ((E.Tagname = 'COMMENT') or (E.Tagname = '!')) and Assigned(FComment) then
                with All.item(x, 0) as IHtmlCommentElement do
                  FComment(self, text, Element) else

                if (E.Tagname = 'IMG') and Assigned(FImage) then
                  with All.item(x, 0) as IHtmlImgElement do
                    FImage(self, src, lowsrc, vrml, dynsrc,
                      alt, align, usemap, isMap, border, loop,
                      vspace, hspace, width, height, Element) else

                  if (E.Tagname = 'BODY') and Assigned(FBody) then
                    with All.item(x, 0) as IHtmlBodyElement do
                      FBody(self, background, bgProperties,
                        leftmargin, topmargin, rightMargin, bottomMargin, bgColor, text, link,
                        vlink, alink, noWrap, Element) else

                    if (E.Tagname = 'BASE') and Assigned(FBase) then
                      with All.item(x, 0) as IHtmlBaseElement do
                        FBase(self, href, target, Element) else

                      if (E.Tagname = 'BASEFONT') and Assigned(FBaseFont) then
                        with All.item(x, 0) as IHtmlBaseFontElement do
                          FBaseFont(self, color, face, size, Element) else

                        if (E.Tagname = 'FONT') and Assigned(FFont) then
                          with All.item(x, 0) as IHtmlFontElement do
                            FFont(self, color, size, face, Element) else

                          if (E.Tagname = 'META') and Assigned(FMeta) then
                            with All.item(x, 0) as IHtmlMEtaElement do
                              FMeta(self, httpEquiv, content, name, url,
                                charset, Element) else

                            if (E.Tagname = 'MARQUEE') and Assigned(FMarquee) then
                              with All.item(x, 0) as IHtmlMarqueeElement do
                                FMarquee(self, bgcolor, width, height, direction, behavior,
                                  ScrollAmount, scrollDelay, loop, vspace, hspace, Element) else

                              if (E.Tagname = 'FORM') and Assigned(FForm) then
                                with All.item(x, 0) as IHtmlFormElement do
                                  FForm(self, action, dir, encoding, method,
                                    target, name, Element) else

                                if (E.Tagname = 'SCRIPT') and Assigned(FScript) then
                                  with All.item(x, 0) as IHtmlScriptElement do
                                    FScript(self, src, htmlFor, event, text, defer, Element) else

                                  if (E.Tagname = 'DIV') and Assigned(FDiv) then
                                    with All.item(x, 0) as IHtmlDivElement do
                                      FDiv(self, align, noWrap, Element) else

                                    if (E.Tagname = 'HR') and Assigned(FHR) then
                                      with All.item(x, 0) as IHtmlHrElement do
                                        FHr(self, align, color, width, size, noShade, Element) else

                                      if (E.Tagname = 'BR') and Assigned(FBR) then
                                        with All.item(x, 0) as IHtmlBrElement do
                                          FBr(self, clear, Element);

          end;
      end
      else
        DispatchMessage(Msg);
    end;

end;



function TIEParser.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
  Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
var
  I: Integer;
begin
  if DispId = DISPID_AMBIENT_DLCONTROL then begin
    I := DLCTL_DOWNLOADONLY + DLCTL_NO_SCRIPTS +
      DLCTL_NO_JAVA + DLCTL_NO_DLACTIVEXCTLS +
      DLCTL_NO_RUNACTIVEXCTLS;
    PVariant(VarResult)^ := I;
    Result := S_OK;
  end else
    Result := DISP_E_MEMBERNOTFOUND;
end;



function TIEParser.OnChanged(dispid: TDispID): HResult;
var
  dp: TDispParams;
  vResult: OleVariant;
begin
  if (DISPID_READYSTATE = Dispid) then
    if SUCCEEDED((Doc as Ihtmldocument2).Invoke(DISPID_READYSTATE, GUID_null,
      LOCALE_SYSTEM_DEFAULT, DISPATCH_PROPERTYGET, dp, @vresult, nil, nil)) then
      if Integer(vresult) = READYSTATE_COMPLETE then
        PostThreadMessage(GetCurrentThreadId(), WM_USER_STARTWALKING, 0, 0);
  RESULT := S_OK;
end;


function TIEParser.LoadUrlFromMoniker: HResult;
var
  moniker: IMoniker;
  BindCtx: IBindCTX;
begin
  createURLMoniker(nil, _Url, Moniker);
  CreateBindCtx(0, BindCtx);
  Result := (Doc as IpersistMoniker).Load(LongBool(0), Moniker, BindCtx, STGM_READ);
end;

function TIEParser.OnRequestEdit(dispid: TDispID): HResult;
begin
  RESULT := E_NOTIMPL;
end;

function TIEParser.SaveObject: HResult;
begin
  result := E_NOTIMPL;
end;

function TIEParser.GetMoniker(dwAssign: Longint; dwWhichMoniker: Longint;
  out mk: IMoniker): HResult;
begin
  result := E_NOTIMPL;
end;

function TIEParser.GetContainer(out container: IOleContainer): HResult;
begin
  result := E_NOTIMPL;
end;

function TIEParser.ShowObject: HResult;
begin
  result := E_NOTIMPL;
end;

function TIEParser.OnShowWindow(fShow: BOOL): HResult;
begin
  result := E_NOTIMPL;
end;

function TIEParser.RequestNewObjectLayout: HResult;
begin
  result := E_NOTIMPL;
end;

end.

