
//***************************************
//       Pluggable protocol demo        *
//        For Delphi 4 og 5             *
//                                      *
//               by                     *
//                                      *
//        Per Linds Larsen             *
//                                      *
//   http://www.euromind.com/iedelphi   *
//***************************************

unit OurProtocol;

interface

uses
  {$IFDEF VER120} d4_app, {$ENDIF} Axctrls, Windows, dialogs, urlmon, sysutils, ActiveX, Classes, ComObj;

const
  Class_Protocol: TGUID = '{3324FA80-A440-11D3-96C7-E5180FEDBD29}';

type
  TProtocol = class(TComObject, IInternetprotocol, IInternetProtocolSink, IInternetBindInfo)
  protected
    IPS : IInternetProtocolSink;
    IBI : IInternetBindInfo;
    FDefaultProtocol: IInternetProtocol;
// IInternetBindInfo
    function GetBindInfo(out grfBINDF: DWORD; var bindinfo: TBindInfo): HResult; stdcall;
    function GetBindString(ulStringType: ULONG; wzStr: POLEStrArray; cEl: ULONG;
      var cElFetched: ULONG): HResult; stdcall;
// IInternetProtocolSink
    function Switch(const ProtocolData: TProtocolData): HResult; stdcall;
    function ReportProgress(ulStatusCode: ULONG; szStatusText: LPCWSTR): HResult; stdcall;
    function ReportData(grfBSCF: DWORD; ulProgress, ulProgressMax: ULONG): HResult; stdcall;
    function ReportResult(hrResult: HResult; dwError: DWORD; szResult: LPCWSTR): HResult; stdcall;
// IInternetprotocol
    function Start(szUrl: LPCWSTR; OIProtSink: IInternetProtocolSink;
      OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult; stdcall;
    function Continue(const ProtocolData: TProtocolData): HResult; stdcall;
    function Abort(hrReason: HResult; dwOptions: DWORD): HResult; stdcall;
    function Terminate(dwOptions: DWORD): HResult; stdcall;
    function Suspend: HResult; stdcall;
    function Resume: HResult; stdcall;
    function Read(pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult; stdcall;
    function Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD; out libNewPosition: ULARGE_INTEGER): HResult; stdcall;
    function LockRequest(dwOptions: DWORD): HResult; stdcall;
    function UnlockRequest: HResult; stdcall;
  public
    function ObjQueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    procedure Initialize; override;

  end;


implementation

uses apphelper, Protocol_unit, ComServ;

{ TProtocol }


function TProtocol.Start(szUrl: LPCWSTR; OIProtSink: IInternetProtocolSink;
  OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult;
var
  s: string;
  w: widestring;
begin
IBI := OIBindInfo;
IPS := OIProtSink;
Trace('');
Trace('START');
Trace('   szUrl: '+szUrl);
  s := szUrl;
  s := copy(s, 3, length(s));
  w := s;
  Result := (FDefaultProtocol as IInternetprotocol).Start(Pwidechar(w), self, self, grfPI, dwReserved);
end;


function TProtocol.Abort(hrReason: HResult; dwOptions: DWORD): HResult;
begin
Trace('');
Trace('ABORT');
Trace('   hrReason: '+InttoStr(hrReason));
Trace('   HResult: '+InttoStr(dwOptions));
  Result := (FDefaultProtocol as IInternetprotocol).Abort(hrReason, dwOptions);
end;

function TProtocol.Continue(const ProtocolData: TProtocolData): HResult;
begin
Trace('');
Trace('CONTINUE');
  Result := (FDefaultProtocol as IInternetprotocol).Continue(ProtocolData);
end;


function TProtocol.LockRequest(dwOptions: DWORD): HResult;
begin
Trace('');
Trace('LOCKREQUEST');
  Result := (FDefaultProtocol as IInternetprotocol).LockRequest(dwOptions);
end;


function TProtocol.Read(pv: Pointer; cb: ULONG;
  out cbRead: ULONG): HResult;
begin
Trace('');
Trace('READ');
Trace('   cb: '+InttoStr(cb));
  Result := (FDefaultProtocol as IInternetprotocol).Read(pv, cb, cbread);
Trace('   cbRead: '+InttoStr(cbRead));
end;

function TProtocol.Resume: HResult;
begin
Trace('');
Trace('RESUME');
  Result := (FDefaultProtocol as IInternetprotocol).Resume;
end;

function TProtocol.Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD;
  out libNewPosition: ULARGE_INTEGER): HResult;
begin
Trace('');
Trace('SEEK');
  Result := (FDefaultProtocol as IInternetprotocol).Seek(dLibMove, dwOrigin, libNewPosition);
end;


function TProtocol.Suspend: HResult;
begin
Trace('');
Trace('SUSPEND');
  Result := (FDefaultProtocol as IInternetprotocol).Suspend;
end;

function TProtocol.Terminate(dwOptions: DWORD): HResult;
begin
Trace('');
Trace('TERMINATE');
  Result := (FDefaultProtocol as IInternetprotocol).Terminate(dwOptions);
IPS._Release;
IBI._Release;
end;

function TProtocol.UnlockRequest: HResult;
begin
Trace('');
Trace('UNLOCKREQUEST');
  Result := (FDefaultProtocol as IInternetprotocol).UnLockRequest;
end;

procedure TProtocol.Initialize;
begin
  inherited;
  if FDefaultProtocol = nil then
    CoCreateInstance(Class_HttpProtocol, Iunknown(Self), CLSCTX_INPROC_SERVER,
      IUnknown, FDefaultProtocol);
end;

function TProtocol.ObjQueryInterface(const IID: TGUID; out Obj): HResult;
begin
  result := inherited ObjQueryInterface(IID, Obj);
  if result = E_NOINTERFACE then Result := FDefaultProtocol.QueryInterface(IID, Obj);
end;

function TProtocol.GetBindInfo(out grfBINDF: DWORD;
  var bindinfo: TBindInfo): HResult;
begin
Trace('');
Trace('GETBINDINFO');
Result:=IBI.GetBindInfo(grfBINDF, bindInfo);
Trace('   grfBINDF: '+TraceBINDF(grfBINDF));
end;

function TProtocol.GetBindString(ulStringType: ULONG; wzStr: POLEStrArray;
  cEl: ULONG; var cElFetched: ULONG): HResult;
begin
Result:=IBI.GetBindString(ulStringType,wzStr, cEl, cElFetched);
Trace('');
Trace('GETBINDSTRING');
Trace('   ulStringType: '+TraceBindString(ulStringType));
Trace('   wzStr: '+wzStr[0]);
end;

function TProtocol.ReportData(grfBSCF: DWORD; ulProgress,
  ulProgressMax: ULONG): HResult;
begin
Trace('');
trace('REPORTDATA');
Trace('   grfBSCF: '+TraceBscf(grfbscf));
Trace('   ulprogress: '+inttostr(ulprogress));
Trace('   ulProgressMax: '+inttostr(ulProgressMax));

result:=IPS.ReportData(grfBSCF, ulProgress, ulProgressMax);
end;

function TProtocol.ReportProgress(ulStatusCode: ULONG;
  szStatusText: LPCWSTR): HResult;
begin
Trace('');
Trace('REPORTPROGRESS');
Trace('   ulStatusCode: '+TraceBindStatus(ulStatusCode));
Trace('   szStatusText: '+szStatusText);
result:=IPS.ReportProgress(ulStatusCode, szStatusText);
end;

function TProtocol.ReportResult(hrResult: HResult; dwError: DWORD;
  szResult: LPCWSTR): HResult;
begin
Trace('');
Trace('REPORTRESULT');
Trace('   hrResult: '+IntToStr(hrResult));
Trace('   dwError: '+IntToStr(dwError));
Trace('   szResult: '+szResult);
result:=IPS.ReportResult(hrresult, dwError, szResult);
end;

function TProtocol.Switch(const ProtocolData: TProtocolData): HResult;
begin
Trace('');
Trace('SWITCH');
result:=IPS.Switch(Protocoldata);
end;

initialization
  TComObjectFactory.Create(ComServer, TProtocol, Class_Protocol,
    'Protocol', '', ciMultiInstance, tmApartment);
end.

