//***********************************************************
//               TSaxContentHandler 1.00 (Jan. 14, 2000)    *
//                                                          *
//                       For Delphi 4/5                     *
//                     Freeware Component                   *
//                            by                            *
//                     Per Linds Larsen                    *
//                   per.lindsoe@larsen.dk                  *
//                                                          *
//  Documentation and updated versions:                     *
//                                                          *
//               http://www.euromind.com/iedelphi           *
//***********************************************************


unit SAXContenthandler;

interface

uses
comobj, activex,  MSXML2_tlb, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

  TstartElementEvent = procedure(var strNamespaceURI: WideString; var strLocalName: WideString;
                           var strQName: WideString; const oAttributes: IVBSAXAttributes) of object;
  TendElementEvent = procedure(var strNamespaceURI: WideString; var strLocalName: WideString;
                         var strQName: WideString) of object;
  TstartDocumentEvent = procedure of object;
  TendDocumentEvent = procedure of object;
  TcharactersEvent = procedure (var strChars: WideString) of object;
  TignorableWhitespaceEvent = procedure(var strChars: WideString) of object;
  TstartPrefixMappingEvent = procedure(var strPrefix: WideString; var strURI: WideString) of object;
  TendPrefixMappingEvent = procedure(var strPrefix: WideString) of object;
  TprocessingInstructionEvent = procedure(var strTarget: WideString; var strData: WideString) of object;
  TskippedEntityEvent = procedure(var strName: WideString) of object;

  TSAXContentHandler = class(TComponent, IUnknown, IVBSAXContentHandler)
  private
    { Private declarations }
    FStartElement: TstartElementEvent;
    FEndElement: TEndElementEvent;
    FCharacters: TcharactersEvent;
    FStartDocument: TstartDocumentEvent;
    FEndDocument: TEndDocumentEvent;
    FIgnorableWhiteSpace: TignorableWhitespaceEvent;
    FStartPrefixMapping: TstartPrefixMappingEvent;
    FEndPrefixMapping: TEndPrefixMappingEvent;
    FProcessingInstruction: TprocessingInstructionEvent;
    FSkippedEntity: TskippedEntityEvent;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Set_documentLocator(const Param1: IVBSAXLocator); safecall;
    procedure startDocument; safecall;
    procedure endDocument; safecall;
    procedure startPrefixMapping(var strPrefix: WideString; var strURI: WideString); safecall;
    procedure endPrefixMapping(var strPrefix: WideString); safecall;
    procedure startElement(var strNamespaceURI: WideString; var strLocalName: WideString;
                           var strQName: WideString; const oAttributes: IVBSAXAttributes); safecall;
    procedure endElement(var strNamespaceURI: WideString; var strLocalName: WideString;
                         var strQName: WideString); safecall;
    procedure characters(var strChars: WideString); safecall;
    procedure ignorableWhitespace(var strChars: WideString); safecall;
    procedure processingInstruction(var strTarget: WideString; var strData: WideString); safecall;
    procedure skippedEntity(var strName: WideString); safecall;
    property documentLocator: IVBSAXLocator write Set_documentLocator;
  published
    { Published declarations }
    property OnStartDocument: TstartDocumentEvent read FStartDocument write FStartDocument;
    property OnEndDocument: TEndDocumentEvent read FEndDocument write FEndDocument;
    property OnStartPrefixMapping: TstartPrefixMappingEvent read FStartPrefixMapping write FStartPrefixMapping;
    property OnEndPrefixMapping: TEndPrefixMappingEvent read FEndPrefixMapping write FEndPrefixMapping;
    property OnStartElement: TstartElementEvent read FStartElement write FStartElement;
    property OnEndElement: TEndElementEvent read FEndElement write FEndElement;
    property OnCharacters: TcharactersEvent read FCharacters write FCharacters;
    property OnIgnorableWhiteSpace: TignorableWhitespaceEvent read FIgnorableWhiteSpace write FIgnorableWhiteSpace;
    property OnProcessingInstruction: TprocessingInstructionEvent read FProcessingInstruction write FProcessingInstruction;
    property OnSkippedEntity: TskippedEntityEvent read FSkippedEntity write FSkippedEntity;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TSAXContentHandler]);
end;

{ TSAXContentHandler }

procedure TSAXContentHandler.characters(var strChars: WideString);
begin
If Assigned(FCharacters) then FCharacters(strChars);
end;

procedure TSAXContentHandler.endDocument;
begin
If Assigned(FendDocument) then FEndDocument;
end;

procedure TSAXContentHandler.endElement(var strNamespaceURI, strLocalName,
  strQName: WideString);
begin
If Assigned(FEndElement) then FEndElement(strNamespaceURI, strLocalName,strQName);
end;

procedure TSAXContentHandler.endPrefixMapping(var strPrefix: WideString);
begin
If Assigned(FEndprefixMapping) then FEndPrefixMapping(StrPrefix);
end;

procedure TSAXContentHandler.ignorableWhitespace(var strChars: WideString);
begin
If Assigned(FignorableWhiteSpace) then FignorableWhiteSpace(StrChars);
end;

procedure TSAXContentHandler.processingInstruction(var strTarget,
  strData: WideString);
begin
If Assigned(FprocessingInstruction) then FprocessingInstruction(strTarget, strData);
end;

procedure TSAXContentHandler.Set_documentLocator(const Param1: IVBSAXLocator);
begin
inherited;
end;

procedure TSAXContentHandler.skippedEntity(var strName: WideString);
begin
If Assigned(FSkippedEntity) then FSkippedEntity(strName);
end;

procedure TSAXContentHandler.startDocument;
begin
If Assigned(FstartDocument) then FStartDocument;
end;

procedure TSAXContentHandler.startElement(var strNamespaceURI, strLocalName,
  strQName: WideString; const oAttributes: IVBSAXAttributes);
begin
If Assigned(FstartElement) then FStartElement(strNamespaceURI, strLocalName,strQName, oAttributes);
end;

procedure TSAXContentHandler.startPrefixMapping(var strPrefix, strURI: WideString);
begin
If Assigned(FstartPrefixMapping) then FstartPrefixMapping(strPrefix, strURI);
end;

end.
