unit Options_u;

interface

uses
  clipbrd, Urlmon, Activex, Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, CheckLst, Spin, ExtCtrls;

type
  TOptionsForm = class(TForm)
    Top: TSpinEdit;
    Left: TSpinEdit;
    Height: TSpinEdit;
    Width: TSpinEdit;
    Options: TCheckListBox;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure TestDialog(ResName: string);
    procedure Button2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    function SetFeatureString: string;
    procedure FormHide(Sender: TObject);
    procedure OptionsClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  OptionsForm: TOptionsForm;
  TopString, LeftString, Edge: string;
  Features: string;

implementation

uses IEResTool_U;

{$R *.DFM}


function TOptionsForm.SetFeatureString: string;
begin
  if Options.Checked[0] then Edge := 'sunken' else Edge := 'raised';
  if Options.Checked[1] then
  begin
    Topstring := '';
    Leftstring := '';
  end else
  begin
    Topstring := InttoStr(Top.Value);
    Leftstring := InttoStr(Left.Value);
  end;
  Result := format('dialogHeight: %dpx; dialogWidth: %dpx; dialogTop:' +
    '%spx; dialogLeft: %spx; center: %d; dialoghide: %d; edge: %s; help: %d;' +
    ' resizable: %d; scroll: %d; status: %d; unadorned: %d;',
    [Height.Value,
    Width.Value,
      topstring,
      leftstring,
      Integer(Options.Checked[1]),
      Integer(Options.Checked[2]),
      Edge,
      Integer(Options.Checked[3]),
      Integer(Options.Checked[4]),
      Integer(Options.Checked[5]),
      Integer(Options.Checked[6]),
      Integer(Options.Checked[7])]);
end;


procedure TOptionsForm.TestDialog(ResName: string);
var
  Url: string;
  Moniker: IMoniker;
  v1, v2: Variant;
begin
  Features := SetFeatureString;
  Url := 'res://' + OutPutdir + Lowercase(OFN) + 'Res.dll/' + Lowercase(Resname);
  CreateUrlMoniker(nil, StringToOleStr(Url), Moniker);
  showhtmldialog(handle, Moniker, v1, StringToOleStr(Features), v2);
end;


procedure TOptionsForm.Button1Click(Sender: TObject);
begin
  Timer1.Enabled := True;
  TestDialog(ResToolForm.ResListbox.Items[ResToolForm.ResListbox.ItemIndex]);
  Timer1.Enabled := False;
end;

procedure TOptionsForm.Button2Click(Sender: TObject);
begin
  ClipBoard.SetTextBuf(Pchar(SetFeatureString));
end;

procedure TOptionsForm.FormShow(Sender: TObject);
begin

  Features := '';
  with ini do begin
    Options.Checked[0] := ReadBool('HTLMDialogOptions', 'Edge', FALSE);
    Options.Checked[1] := ReadBool('HTLMDialogOptions', 'Center', TRUE);
    Options.Checked[2] := ReadBool('HTLMDialogOptions', 'DialogHide', FALSE);
    Options.Checked[3] := ReadBool('HTLMDialogOptions', 'Help', TRUE);
    Options.Checked[4] := ReadBool('HTLMDialogOptions', 'Resizable', FALSE);
    Options.Checked[5] := ReadBool('HTLMDialogOptions', 'Scroll', TRUE);
    Options.Checked[6] := ReadBool('HTLMDialogOptions', 'Status', TRUE);
    Options.Checked[7] := ReadBool('HTLMDialogOptions', 'Unadorned', FALSE);
    height.Value := ReadInteger('HTLMDialogOptions', 'Height', 155);
    width.Value := ReadInteger('HTLMDialogOptions', 'Width', 600);
    top.Value := ReadInteger('HTLMDialogOptions', 'Top', 200);
    left.Value := ReadInteger('HTLMDialogOptions', 'Left', 200);
  end;
end;

procedure TOptionsForm.FormHide(Sender: TObject);
begin
  with ini do begin
    Writebool('HTLMDialogOptions', 'Edge', Options.Checked[0]);
    Writebool('HTLMDialogOptions', 'Center', Options.Checked[1]);
    Writebool('HTLMDialogOptions', 'DialogHide', Options.Checked[2]);
    Writebool('HTLMDialogOptions', 'Help', Options.Checked[3]);
    Writebool('HTLMDialogOptions', 'Resizable', Options.Checked[4]);
    Writebool('HTLMDialogOptions', 'Scroll', Options.Checked[5]);
    Writebool('HTLMDialogOptions', 'Status', Options.Checked[6]);
    Writebool('HTLMDialogOptions', 'Unadorned', Options.Checked[7]);
    WriteInteger('HTLMDialogOptions', 'Height', height.Value);
    WriteInteger('HTLMDialogOptions', 'Width', width.Value);
    WriteInteger('HTLMDialogOptions', 'Top', top.Value);
    WriteInteger('HTLMDialogOptions', 'Left', left.Value);
  end;
end;

procedure TOptionsForm.OptionsClick(Sender: TObject);
begin
  with Options do begin
    case Itemindex of
      0: hint := 'Specifies the edge style of the dialog window. The default is raised.';
      1: hint := 'Specifies whether to center the dialog window within the desktop.' + #13 +
        'the default is yes.';
      2: hint := 'Specifies whether the dialog window is hidden when printing' + #13 +
        'or using print preview. This feature is only available when' + #13 +
          'a dialog box is opened from a trusted application. The default is no.';
      3: hint := 'Specifies whether the dialog window displays the context-sensitive' + #13 +
        'Help icon. The default is yes';
      4: hint := 'Specifies whether the dialog window has fixed dimensions. The default is no';
      5: hint := 'Specifies whether the dialog window displays scrollbars. The default is yes.';
      6: hint := 'Specifies whether the dialog window displays a status bar. The default' + #13 +
        'is yes for untrusted dialog windows and no for trusted dialog windows';
      7: hint := 'Specifies whether the dialog window displays the border window chrome.' + #13 +
        'This feature is only available when a dialog box is opened from a trusted' + #13 +
          'application. The default is no.'
    end;
  end;
end;

procedure TOptionsForm.Timer1Timer(Sender: TObject);
var
  rect: TRect;
begin
  getwindowrect(GetActiveWindow, Rect);
  top.value := rect.Top;
  left.Value := rect.Left;
  Height.Value := Rect.Bottom - Rect.Top;
  Width.Value := Rect.Right - Rect.Left;
end;

end.

