unit main;

(*

  Last Modified : 1-8-2000
  Author        : Michael Cessna <mcessna@saleslogix.com>
  Purpose       : To demonstrate the TWebBrowser component.

  This project was created using Delphi 5. Its purpose is to provide a
  starting point for working with the TWebBrowser component. As such,
  the code is incomplete, and should be considered Beta. Use this software
  at your own risk.
  
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ToolWin, OleCtrls, ImgList, Menus, Buttons, ShlObj, StdCtrls,
  IEAddress, ShellAPI, BrowseDr, ExtCtrls, IniFiles, {$IFDEF VER130}SHDocVw
  {$ELSE}SHDocVw_TLB{$ENDIF};

type
  TFormMain = class(TForm)
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ToolBar1: TToolBar;
    TabSheet3: TTabSheet;
    WB2: TWebBrowser;
    WB3: TWebBrowser;
    ImageListHot: TImageList;
    ImageListDisabled: TImageList;
    BTN_1_STOP: TToolButton;
    BTN_1_REFRESH: TToolButton;
    BTN_1_HOME: TToolButton;
    BTN_1_SEARCH: TToolButton;
    BTN_1_FAVORITES: TToolButton;
    BTN_1_MAIL: TToolButton;
    BTN_1_PRINT: TToolButton;
    BTN_1_EDIT: TToolButton;
    ToolButton13: TToolButton;
    ToolButton14: TToolButton;
    ToolButton15: TToolButton;
    ImageListDefault: TImageList;
    BTN_1_BACK: TToolButton;
    BTN_1_FORWARD: TToolButton;
    ToolBar2: TToolBar;
    BTN_2_BACK: TToolButton;
    BTN_2_FORWARD: TToolButton;
    ToolButton18: TToolButton;
    BTN_2_STOP: TToolButton;
    BTN_2_REFRESH: TToolButton;
    BTN_2_HOME: TToolButton;
    ToolButton22: TToolButton;
    BTN_2_SEARCH: TToolButton;
    BTN_2_FAVORITES: TToolButton;
    ToolButton26: TToolButton;
    BTN_2_MAIL: TToolButton;
    BTN_2_PRINT: TToolButton;
    BTN_2_EDIT: TToolButton;
    MainMenu: TMainMenu;
    MNU_FILE: TMenuItem;
    ITM_FILE_EXIT: TMenuItem;
    ToolBar3: TToolBar;
    BTN_3_BACK: TToolButton;
    BTN_3_FORWARD: TToolButton;
    ToolButton19: TToolButton;
    BTN_3_STOP: TToolButton;
    BTN_3_REFRESH: TToolButton;
    BTN_3_HOME: TToolButton;
    ToolButton24: TToolButton;
    BTN_3_SEARCH: TToolButton;
    BTN_3_FAVORITES: TToolButton;
    ToolButton29: TToolButton;
    BTN_3_MAIL: TToolButton;
    BTN_3_PRINT: TToolButton;
    BTN_3_EDIT: TToolButton;
    CoolBar2: TCoolBar;
    IEAddress2: TIEAddress;
    CoolBar1: TCoolBar;
    IEAddress1: TIEAddress;
    CoolBar3: TCoolBar;
    IEAddress3: TIEAddress;
    BrowseDlg: TdfsBrowseDirectoryDlg;
    WB1: TWebBrowser;
    StatusBar: TStatusBar;
    procedure ITM_FILE_EXITClick(Sender: TObject);
    procedure HandleWebBrowserButtonClick(Sender: TObject);
    procedure WebBrowserCommandStateChange(Sender: TObject;
      Command: Integer; Enable: WordBool);
    procedure IEAddressUrlSelected(Url: String);
    procedure WebBrowserBeforeNavigate2(Sender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure WebBrowserStatusTextChange(Sender: TObject;
      const Text: WideString);
    procedure WebBrowserDocumentComplete(Sender: TObject;
      const pDisp: IDispatch; var URL: OleVariant);
    procedure FormCreate(Sender: TObject);
    procedure BrowseDlgSelChanged(Sender: TObject; NewSel: String;
      NewSelPIDL: PItemIDList);
    procedure PageControlChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure PageControlChange(Sender: TObject);
    procedure IEAddress1UrlSelected(Sender: TObject; Url: String);
  private
    { Private declarations }
    procedure ChangeEnableButtons(WB: TWebBrowser);
    procedure EditWebBrowserDoc(WB: TWebBrowser);
    function GetVisibleWebBrowser: TWebBrowser;
    function Go(WB: TWebBrowser; const URL: String): Boolean;
    procedure PrintWebBrowser(WB: TWebBrowser);
    procedure SendEmailLink(WB: TWebBrowser);
    procedure ShowFavoritesDlg(WB: TWebBrowser);
  public
    { Public declarations }
  end;

var
  FormMain : TFormMain;

implementation

uses ActiveX, intercept;

{$R *.DFM}

procedure HandleWin32Error(eMsg: String; eResult: Cardinal);
begin
  eMsg := eMsg + SysErrorMessage( eResult );
  raise Exception.Create( eMsg );
end;

{$HINTS OFF}
function TFormMain.Go(WB: TWebBrowser; const URL: String): Boolean;
var
  eMsg : String;
begin

  Result := False;

  try
    WB.Navigate( URL );
    Result := True;
  except
    on E: Exception do
    begin
      eMsg := 'Unable to navigate to ' + URL + '. ' + E.Message + '.';
      MessageBeep( MB_ICONERROR );
      raise Exception.Create( eMsg );
    end;
  end;

end;
{$HINTS ON}

function TFormMain.GetVisibleWebBrowser: TWebBrowser;
var
  I  : Integer;
  WB : TWebBrowser;
begin

  Result := nil;

  for I := 0 to ComponentCount -1 do
  begin
    if Components[I].ClassName = 'TWebBrowser' then
    begin
      WB := TWebBrowser( Components[I] );
      if IsWindowVisible( WB.Handle ) then
      begin
        Result := WB;
      end;
    end;
  end;

end;

procedure TFormMain.ChangeEnableButtons(WB: TWebBrowser);
var
  ChangeEnable : Boolean;
  IsEditable   : Boolean;
  Protocol     : String;
begin
  
  try
    ChangeEnable := WB.OleObject.document.queryCommandSupported( 'Print' );
  except
    ChangeEnable := False;
  end;

  try
    Protocol := UpperCase( WB.OleObject.document.location.protocol );
  except
    Protocol := '';
  end;

  IsEditable := ( Protocol = 'FILE:' ) or
                ( Protocol = 'HTTP:' ) or
                ( Protocol = 'HTTPS:' );

  case WB.Tag of
    1 :
    begin
      BTN_1_MAIL.Enabled  := ChangeEnable;
      BTN_1_PRINT.Enabled := ChangeEnable;
      BTN_1_EDIT.Enabled  := not WB.Busy and IsEditable;
    end;

    2 :
    begin
      BTN_2_MAIL.Enabled  := ChangeEnable;
      BTN_2_PRINT.Enabled := ChangeEnable;
      BTN_2_EDIT.Enabled  := not WB.Busy and IsEditable;
    end;

    3 :
    begin
      BTN_3_MAIL.Enabled  := ChangeEnable;
      BTN_3_PRINT.Enabled := ChangeEnable;
      BTN_3_EDIT.Enabled  := not WB.Busy and IsEditable;
    end;
  end;

end;

procedure TFormMain.EditWebBrowserDoc(WB: TWebBrowser);
var
  Protocol : String;
  URL      : String;
  RetVal   : Integer;
  vaIn     : OleVariant;
  vaOut    : OleVariant;
  W        : Word;
begin

  (*
    Assumptions: The user has an Edit command associated with HTML files.
    Recommendation: Allow the user to define their own editor, and use that.

    NOTE: If your editor does not come up correctly, if may not be configured
          properly. Check the HTML association with the Edit verb.
  *)

  if WB.Busy then
  begin
    MessageDlg( 'Unable to process your request. The Web browser is busy.',
                mtInformation,
                [mbOK],
                0
    );
    Exit;
  end;

  URL := WB.LocationURL;

  Protocol := UpperCase( WB.OleObject.document.location.protocol );

  if ( Protocol = 'HTTP:' ) or ( Protocol = 'HTTPS:' ) then
  begin

    MessageBeep( MB_ICONINFORMATION );
    W := MessageDlg( 'This page must be saved to your computer ' +
                     'before you can edit it. Okay to save?',
                     mtConfirmation,
                     [mbYes, mbNo],
                     0
    );
    case W of
      mrYes : FormIntercept.Timer.Enabled := True;
      mrNo  : Exit;
    end;
    
    try
      WB.ControlInterface.ExecWB( OLECMDID_SAVEAS,
                                  OLECMDEXECOPT_PROMPTUSER,
                                  vaIn,
                                  vaOut
      );
    except
      on E: Exception do
      begin
        FormIntercept.Timer.Enabled := False;
        MessageBeep( MB_ICONERROR );
        MessageDlg( 'Unable to save page. ' + E.Message + '.',
                    mtError,
                    [mbOK],
                    0
        );
        Exit;
      end;
    end;

    if WasSaved then
    begin

      { wait until done downloading }
      while WB.Busy do Application.ProcessMessages;

      if FileExists( SavedFileName ) then
      begin
        
        RetVal := ShellExecute( Handle,
                                'Edit',
                                PChar( SavedFileName ),
                                nil,
                                nil,
                                SW_SHOWNORMAL
        );
        if RetVal <= 32 then
        begin
          HandleWin32Error( 'Error opening ' + SavedFileName + ' for editing. ',
            GetLastError );
        end;
      end
      else
      begin
        { you could allow the user to select the file, and then launch it }
        MessageDlg( 'The file ' + SavedFileName + ' could not be found.',
          mtError, [mbOK], 0 );
      end;

    end;

  end;

  if Protocol = 'FILE:' then
  begin
    URL := StringReplace( URL, '%20', ' ', [rfReplaceAll] );
    RetVal := ShellExecute( Handle,
                            'Edit',
                            PChar( URL ),
                            nil,
                            nil,
                            SW_SHOWNORMAL
    );
    if RetVal <= 32 then
    begin
      HandleWin32Error( 'Error opening ' + URL + ' for editing. ',
        GetLastError );
    end;
  end;

end;

procedure TFormMain.ShowFavoritesDlg(WB: TWebBrowser);
var
  Ini : TIniFile;
  URL : String;
begin
  with BrowseDlg do
  begin
    Caption := 'Internet Favorites';
    Title := 'Select a favorite:';
    Root := idFavorites;
    if Execute then
    begin
      Ini := TIniFile.Create( Selection );
      try
        URL := Ini.ReadString( 'InternetShortcut', 'URL', '' );
        Go( WB, URL );
      finally
        Ini.Free;
      end;
    end;
  end;
end;

procedure TFormMain.PrintWebBrowser(WB: TWebBrowser);
begin
  try
    WB.OleObject.document.execCommand( 'Print' );
  except
    on E: Exception do
    begin
      MessageBeep( MB_ICONERROR );
      MessageDlg( 'Error printing. ' + E.Message + '.', mtError, [mbOK], 0 );
    end;
  end;
end;

procedure TFormMain.SendEmailLink(WB: TWebBrowser);
var
  URL    : String;
  Title  : String;
  RetVal : Integer;
begin

  URL := WB.LocationURL;

  Title := WB.LocationName;

  RetVal := ShellExecute( Handle,
                          'open',
                          PChar( 'mailto:?body=' + Title + ' ' + URL ),
                          nil,
                          nil,
                          SW_SHOWNORMAL
  );
  if RetVal <= 32 then
  begin
    HandleWin32Error( 'Error: ', GetLastError );
  end;

end;

procedure TFormMain.HandleWebBrowserButtonClick(Sender: TObject);
var
  WB : TWebBrowser;
  ID : Integer;
begin

  ID := TSpeedButton( Sender ).Tag;

  WB := GetVisibleWebBrowser;

  if WB <> nil then
  begin
    case ID of
      10  : try WB.GoBack except end;    // Back
      20  : try WB.GoForward except end; // Froward
      30  : WB.Stop;                     // Stop
      40  : WB.Refresh;                  // Refresh
      50  : WB.GoHome;                   // Home
      60  : WB.GoSearch;                 // Search
      70  : ShowFavoritesDlg(WB);        // Favorites
      80  : SendEmailLink(WB);           // Mail
      90  : PrintWebBrowser(WB);         // Print
      100 : EditWebBrowserDoc(WB);       // Edit
    end;
  end;

end;

procedure TFormMain.ITM_FILE_EXITClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.WebBrowserCommandStateChange(Sender: TObject;
  Command: Integer; Enable: WordBool);
var
  ID : Integer;
begin

  if IsWindowVisible( Handle ) then
  begin
    ChangeEnableButtons( TWebBrowser( Sender ) );
  end;

  ID := TWebBrowser( Sender ).Tag;

  case ID of
    1 : { WB1 }
    begin
      case Command of
        CSC_NAVIGATEBACK :
        begin
          BTN_1_BACK.Enabled := Enable;
        end;

        CSC_NAVIGATEFORWARD :
        begin
          BTN_1_FORWARD.Enabled := Enable;
        end;
      end;
    end;

    2 : { WB2 }
    begin
      case Command of
        CSC_NAVIGATEBACK :
        begin
          BTN_2_BACK.Enabled := Enable;
        end;

        CSC_NAVIGATEFORWARD :
        begin
          BTN_2_FORWARD.Enabled := Enable;
        end;
      end;
    end;

    3 : { WB3 }
    begin
      case Command of
        CSC_NAVIGATEBACK :
        begin
          BTN_3_BACK.Enabled := Enable;
        end;

        CSC_NAVIGATEFORWARD :
        begin
          BTN_3_FORWARD.Enabled := Enable;
        end;
      end;
    end;

  end;

end;

procedure TFormMain.IEAddressUrlSelected(Url: String);
begin;
end;

procedure TFormMain.WebBrowserBeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var
  ID      : Integer;
  Address : TIEAddress;
begin

  ID := TWebBrowser( Sender ).Tag;

  Address := nil;

  case ID of
    1 : Address := IEAddress1;
    2 : Address := IEAddress2;
    3 : Address := IEAddress3;
  end;

  if Assigned( Address ) then
  begin
    if ID > 0 then
    begin
      if ( TargetFrameName = '' ) and
         ( UpperCase( URL ) <> UpperCase( Address.URL ) ) then
      begin
        Address.URL := URL;
      end;
    end;
  end;

end;

procedure TFormMain.WebBrowserStatusTextChange(Sender: TObject;
  const Text: WideString);
begin
  if not IsWindowVisible( Handle ) then Exit;
  StatusBar.SimpleText := Text;
end;

procedure TFormMain.WebBrowserDocumentComplete(Sender: TObject;
  const pDisp: IDispatch; var URL: OleVariant);
var
  WB : TWebBrowser;
begin

  WB := TWebBrowser( Sender );

  { if the entire document is done downloading }
  if pDisp = WB.DefaultDispatch then
  begin
    ChangeEnableButtons(WB);
  end;

end;

procedure TFormMain.FormCreate(Sender: TObject);
var
  I  : Integer;
  WB : TWebBrowser;
begin
  for I := 0 to ComponentCount -1 do
  begin
    if Components[I].ClassName = 'TWebBrowser' then
    begin
      WB := TWebBrowser( Components[I] );
      if not Go( WB, 'about:blank' ) then Break;
    end;
  end;
end;

procedure TFormMain.BrowseDlgSelChanged(Sender: TObject; NewSel: String;
  NewSelPIDL: PItemIDList);
begin
  BrowseDlg.EnableOKButton := ExtractFileExt( UpperCase( NewSel ) ) = '.URL';
end;

procedure TFormMain.PageControlChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  LockWindowUpdate( Handle );
end;

procedure TFormMain.PageControlChange(Sender: TObject);
begin
  Refresh;
  LockWindowUpdate(0);
end;

procedure TFormMain.IEAddress1UrlSelected(Sender: TObject; Url: String);
var
  WB : TWebBrowser;
begin
  WB := GetVisibleWebBrowser;
  if WB <> nil then Go( WB, Url );
end;

initialization
  OleInitialize(nil);

finalization
  OleUninitialize;

end.


