unit _[NAME]Protocol;

interface

uses
 Comserv, Registry, SysUtils, ComObj, Classes, AxCtrls, Windows, ActiveX, Urlmon;

const
  Class_[NAME]Protocol: TGUID = '[GUID]';

type
  T[NAME]ProtocolFactory = class(TComObjectFactory)
  private
    procedure AddKeys;
    procedure RemoveKeys;
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

type
  T[NAME]Protocol = class(TComObject, IInternetProtocol)
  protected
    DataStream: IStream;
    TotalSize: Integer;
    Written: Integer;
    function Start(szUrl: LPCWSTR; OIProtSink: IInternetProtocolSink;
      OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult; stdcall;
    function Continue(const ProtocolData: TProtocolData): HResult; stdcall;
    function Abort(hrReason: HResult; dwOptions: DWORD): HResult; stdcall;
    function Terminate(dwOptions: DWORD): HResult; stdcall;
    function Suspend: HResult; stdcall;
    function Resume: HResult; stdcall;
    function Read(pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult; stdcall;
    function Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD; out libNewPosition: ULARGE_INTEGER): HResult; stdcall;
    function LockRequest(dwOptions: DWORD): HResult; stdcall;
    function UnlockRequest: HResult; stdcall;
  end;



implementation

{$R [NAME]Res.RES}

var
  h: HInst;

function T[NAME]Protocol.Start(szUrl: LPCWSTR; OIProtSink: IInternetProtocolSink;
  OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult;
var
  Url: string;
  h2: HInst;
  Data: Pointer;
  Dummy: Int64;
begin

  Totalsize := 0;
  written := 0;
  url := (Copy(szurl, Pos(':', SzUrl) + 1, 255));
  h2 := Findresource(h, Pchar(url), '#23');
  TotalSize := SizeOfResource(h, h2);
  Data := LockResource(LoadResource(h, h2));
  CreateStreamOnHGlobal(0, True, DataStream);
  TOleStream.Create(DataStream).WriteBuffer(Data^, TotalSize);
  DataStream.Seek(0, STREAM_SEEK_SET, Dummy);
  OIProtSink.ReportData(BSCF_FIRSTDATANOTIFICATION or
    BSCF_LASTDATANOTIFICATION or BSCF_DATAFULLYAVAILABLE, TotalSize, TotalSize);
  OIProtSink.ReportResult(S_OK, 0, nil);
  Result := S_OK;
end;

function T[NAME]Protocol.Read(pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult;
begin
  DataStream.Read(pv, cb, @cbRead);
  Inc(written, cbread);
  if (written = totalSize) then
    result := S_FALSE else Result := S_OK;
end;


function T[NAME]Protocol.Abort(hrReason: HResult; dwOptions: DWORD): HResult;
begin
  Result := S_OK;
end;

function T[NAME]Protocol.Continue(const ProtocolData: TProtocolData): HResult;
begin
  Result := S_OK;
end;

function T[NAME]Protocol.LockRequest(dwOptions: DWORD): HResult;
begin
  Result := S_OK;
end;


function T[NAME]Protocol.Resume: HResult;
begin
  Result := S_OK;
end;

function T[NAME]Protocol.Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD;
  out libNewPosition: ULARGE_INTEGER): HResult;
begin
  Result := S_OK;
end;

function T[NAME]Protocol.Suspend: HResult;
begin
  Result := S_OK;
end;

function T[NAME]Protocol.Terminate(dwOptions: DWORD): HResult;
begin
  result := S_OK;
end;

function T[NAME]Protocol.UnlockRequest: HResult;
begin
  Result := S_OK;
end;



procedure T[NAME]ProtocolFactory.UpdateRegistry(Register: Boolean);
begin
  inherited UpdateRegistry(Register);
  if Register then AddKeys else RemoveKeys;
end;

procedure T[NAME]ProtocolFactory.AddKeys;
var S: string;
begin
  S := GUIDToString(CLASS_[NAME]Protocol);
  with TRegistry.Create do
  try
    RootKey := HKEY_CLASSES_ROOT;
    if OpenKey('PROTOCOLS\Handler\[NAME]', True) then
    begin
      WriteString('', '[NAME]');
      WriteString('CLSID', S);
      CloseKey;
    end;
  finally
    Free;
  end;
end;

procedure T[NAME]ProtocolFactory.RemoveKeys;
var S: string;
begin
  S := GUIDToString(CLASS_[NAME]Protocol);
  with TRegistry.Create do
  try
    RootKey := HKEY_CLASSES_ROOT;
    DeleteKey('PROTOCOLS\Handler\[NAME]');
  finally
    Free;
  end;
end;

initialization

h := LoadLibrary('[NAME]Protocol.dll');

  T[NAME]ProtocolFactory.Create(ComServer, T[NAME]Protocol, Class_[NAME]Protocol,
    '[NAME]Protocol', '', ciMultiInstance, tmApartment);

finalization

freelibrary(h);

end.

