//***********************************************************
//               TXMLParser ver 1.00 (Jan. 14, 2000)        *
//                                                          *
//                       For Delphi 5                       *
//                     Freeware Component                   *
//                            by                            *
//                     Per Linds Larsen                    *
//                   per.lindsoe@larsen.dk                  *
//                                                          *
//  Documentation and updated versions:                     *
//                                                          *
//               http://www.euromind.com/iedelphi           *
//***********************************************************


unit xmlparser;

interface

uses
contnrs, MsXml2_tlb, comobj, Activex, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

  TAttribute = class(TObject)
  public
    uri: string;
    LocalName: string;
    QName: string;
    aType: string;
    value: string;
  end;

  TXMLAttributes = class(TInterfacedObject, IVBSaxAttributes)
  private
    AttrList: TObjectList;
    { Private declarations }
    function findIndexByNSName(strURI, strLocalName: string): Integer;
    function findIndexByQName(strQName: string): Integer;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    destructor destroy; override;
    procedure clear;
    function CreateAttribute(uri, localName, qName, aType, value: string): TAttribute;
    procedure addAttribute(uri, localName, qName, aType, value: string);
    procedure setAttribute(index: Integer; uri, localName, qName, aType, Value: string);
    procedure SetAttributes(atts: IVBSAXAttributes);
    procedure removeAttribute(index: Integer);
    function Get_length: SYSINT; safecall;
    function getURI(nIndex: SYSINT): WideString; safecall;
    function getLocalName(nIndex: SYSINT): WideString; safecall;
    function getQName(nIndex: SYSINT): WideString; safecall;
    function getIndexFromName(const strURI: WideString; const strLocalName: WideString): SYSINT; safecall;
    function getIndexFromQName(const strQName: WideString): SYSINT; safecall;
    function getType(nIndex: SYSINT): WideString; safecall;
    function getTypeFromName(const strURI: WideString; const strLocalName: WideString): WideString; safecall;
    function getTypeFromQName(const strQName: WideString): WideString; safecall;
    function getValue(nIndex: SYSINT): WideString; safecall;
    function getValueFromName(const strURI: WideString; const strLocalName: WideString): WideString; safecall;
    function getValueFromQName(const strQName: WideString): WideString; safecall;
    property length: SYSINT read Get_length;
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;

  published
    { Published declarations }
  end;


// IVBSAXContentHandler
  TstartElementEvent = procedure(var strNamespaceURI: WideString; var strLocalName: WideString;
    var strQName: WideString; const oAttributes: IVBSAXAttributes) of object;
  TendElementEvent = procedure(var strNamespaceURI: WideString; var strLocalName: WideString;
    var strQName: WideString) of object;
  TstartDocumentEvent = procedure of object;
  TendDocumentEvent = procedure of object;
  TcharactersEvent = procedure(var strChars: WideString) of object;
  TignorableWhitespaceEvent = procedure(var strChars: WideString) of object;
  TstartPrefixMappingEvent = procedure(var strPrefix: WideString; var strURI: WideString) of object;
  TendPrefixMappingEvent = procedure(var strPrefix: WideString) of object;
  TprocessingInstructionEvent = procedure(var strTarget: WideString; var strData: WideString) of object;
  TskippedEntityEvent = procedure(var strName: WideString) of object;
// IVBSAXDTDHandler
  TnotationDeclEvent = procedure(var strName: WideString; var strPublicId: WideString;
    var strSystemId: WideString) of object;
  TunparsedEntityDeclEvent = procedure(var strName: WideString; var strPublicId: WideString;
    var strSystemId: WideString; var strNotationName: WideString) of object;
// IVBSAXDeclHandler
  TelementDeclEvent = procedure(var strName: WideString; var strModel: WideString) of object;
  TattributeDeclEvent = procedure(var strElementName: WideString; var strAttributeName: WideString;
    var strType: WideString; var strValueDefault: WideString;
    var strValue: WideString) of object;
  TinternalEntityDeclEvent = procedure(var strName: WideString; var strValue: WideString) of object;
  TExternalEntityDeclEvent = procedure(var strName: WideString; var strPublicId: WideString;
    var strSystemId: WideString) of object;
// IVBSAXLexicalHandler
  TcommentEvent = procedure(var strChars: WideString) of object;
  TstartCDATAEvent = procedure of object;
  TendCDATAEvent = procedure of object;
  TendDTDEvent = procedure of object;
  TstartDTDEvent = procedure(var strName: WideString; var strPublicId: WideString;
    var strSystemId: WideString) of object;
  TstartEntityEvent = procedure(var strName: WideString) of object;
  TendEntityEvent = procedure(var strName: WideString) of object;
// IVBSAXErrorHandler
  TErrorEvent = procedure(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
    nErrorCode: Integer) of object;
  TfatalErrorEvent = procedure(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
    nErrorCode: Integer) of object;
  TignorableWarningEvent = procedure(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
    nErrorCode: Integer) of object;


  TXMLParser = class(TComponent, IUnknown, IVBSAXXMLReader, IVBSaxXMLFilter, IVBSAXContenthandler,
      IVBSaxErrorHandler, IVBSaxDTDHandler, IVBSaxLexicalHandler, IVBSaxDeclHandler)

  private
    FWriting: Boolean;
    Reader: IVBSAXXMLReader;
    Writer: IMXWriter;
// IVBSAXContentHandler
    FStartElement: TstartElementEvent;
    FEndElement: TEndElementEvent;
    FCharacters: TcharactersEvent;
    FStartDocument: TstartDocumentEvent;
    FEndDocument: TEndDocumentEvent;
    FIgnorableWhiteSpace: TignorableWhitespaceEvent;
    FStartPrefixMapping: TstartPrefixMappingEvent;
    FEndPrefixMapping: TEndPrefixMappingEvent;
    FProcessingInstruction: TprocessingInstructionEvent;
    FSkippedEntity: TskippedEntityEvent;
// IVBSAXDTDHandler
    FNotationDecl: TnotationDeclEvent;
    FUnparsedEntityDecl: TunparsedEntityDeclEvent;
// IVBSAXDeclHandler
    FElementDecl: TelementDeclEvent;
    FAttributeDecl: TattributeDeclEvent;
    FInternalEntityDecl: TinternalEntityDeclEvent;
    FExternalEntityDecl: TExternalEntityDeclEvent;
// IVBSAXLexicalHandler
    FStartDTD: TstartDTDEvent;
    FEndDTD: TendDTDEvent;
    FStartEntity: TstartEntityEvent;
    FEndEntity: TendEntityEvent;
    FStartCData: TstartCDATAEvent;
    FEndCData: TendCDATAEvent;
    FComment: TcommentEvent;
//IVBSaxErrorHandler
    FError: TerrorEvent;
    FFatalError: TFatalerrorEvent;
    FIgnorableWarning: TignorableWarningEvent;

    FErrorHandler: IVBSaxErrorHandler;
    FContentHandler: IVBSAXContenthandler;
    FDTDHandler: IVBSAXDTDHandler;
    FLexicalHandler: IVBSaxLexicalHandler;
    FDeclHandler: IVBSaxDeclHandler;

    WErrorHandler: IVBSaxErrorHandler;
    WContentHandler: IVBSAXContenthandler;
    WDTDHandler: IVBSAXDTDHandler;
    WLexicalHandler: IVBSaxLexicalHandler;
    WDeclHandler: IVBSaxDeclHandler;


    { Private declarations }
  protected
    { Protected declarations }

//IVBSaxXMLFilter
    function Get_parent: IVBSAXXMLReader; safecall;
    procedure Set_parent(const oReader: IVBSAXXMLReader); safecall;
//IVBReader
    function Get_entityResolver: IVBSAXEntityResolver; safecall;
    procedure Set_entityResolver(const oResolver: IVBSAXEntityResolver); safecall;
    function Get_contentHandler: IVBSAXContentHandler; safecall;
    procedure Set_contentHandler(const oHandler: IVBSAXContentHandler); safecall;
    function Get_dtdHandler: IVBSAXDTDHandler; safecall;
    procedure Set_dtdHandler(const oHandler: IVBSAXDTDHandler); safecall;
    function Get_errorHandler: IVBSAXErrorHandler; safecall;
    procedure Set_errorHandler(const oHandler: IVBSAXErrorHandler); safecall;
    function Get_baseURL: WideString; safecall;
    procedure Set_baseURL(const strBaseURL: WideString); safecall;
    function Get_secureBaseURL: WideString; safecall;
    procedure Set_secureBaseURL(const strSecureBaseURL: WideString); safecall;
//IVBWriter
    procedure Set_output(varDestination: OleVariant);
    function Get_output: OleVariant;
    procedure Set_encoding(const strEncoding: WideString);
    function Get_encoding: WideString;
    procedure Set_byteOrderMark(fWriteByteOrderMark: Boolean);
    function Get_byteOrderMark: Boolean;
    procedure Set_indent(fIndentMode: WordBool);
    function Get_indent: WordBool;
    procedure Set_standalone(fValue: WordBool);
    function Get_standalone: WordBool;
    procedure Set_omitXMLDeclaration(fValue: WordBool);
    function Get_omitXMLDeclaration: WordBool;
    procedure Set_version(const strVersion: WideString);
    function Get_version: WideString;
    procedure Set_disableOutputEscaping(fValue: WordBool);
    function Get_disableOutputEscaping: WordBool;

    function GetbaseURL: WideString;
    procedure SetbaseURL(const strBaseURL: WideString);
    function GetsecureBaseURL: WideString;
    procedure SetsecureBaseURL(const strSecureBaseURL: WideString);

  public
    Locator: IVBSAXLocator;
    { Public declarations }
    constructor Create(Owner: TComponent); override;
    //IVBSaxLexicalHandler
    procedure startDTD(var strName: WideString; var strPublicId: WideString;
      var strSystemId: WideString); safecall;
    procedure endDTD; safecall;
    procedure startEntity(var strName: WideString); safecall;
    procedure endEntity(var strName: WideString); safecall;
    procedure startCDATA; safecall;
    procedure endCDATA; safecall;
    procedure comment(var strChars: WideString); safecall;
//IVBSaxErrorhandler
    procedure error(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
      nErrorCode: Integer); safecall;
    procedure fatalError(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
      nErrorCode: Integer); safecall;
    procedure ignorableWarning(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
      nErrorCode: Integer); safecall;
//IVBSaxDTDHandler
    procedure notationDecl(var strName: WideString; var strPublicId: WideString;
      var strSystemId: WideString); safecall;
    procedure unparsedEntityDecl(var strName: WideString; var strPublicId: WideString;
      var strSystemId: WideString; var strNotationName: WideString); safecall;
//IVBSaxDeclHandler
    procedure elementDecl(var strName: WideString; var strModel: WideString); safecall;
    procedure attributeDecl(var strElementName: WideString; var strAttributeName: WideString;
      var strType: WideString; var strValueDefault: WideString;
      var strValue: WideString); safecall;
    procedure internalEntityDecl(var strName: WideString; var strValue: WideString); safecall;
    procedure externalEntityDecl(var strName: WideString; var strPublicId: WideString;
      var strSystemId: WideString); safecall;
//IVBSaxContentHandler
    procedure Set_documentLocator(const Param1: IVBSAXLocator); safecall;
    procedure startDocument; safecall;
    procedure endDocument; safecall;
    procedure startPrefixMapping(var strPrefix: WideString; var strURI: WideString); safecall;
    procedure endPrefixMapping(var strPrefix: WideString); safecall;
    procedure startElement(var strNamespaceURI: WideString; var strLocalName: WideString;
      var strQName: WideString; const oAttributes: IVBSAXAttributes); safecall;
    procedure endElement(var strNamespaceURI: WideString; var strLocalName: WideString;
      var strQName: WideString); safecall;
    procedure characters(var strChars: WideString); safecall;
    procedure ignorableWhitespace(var strChars: WideString); safecall;
    procedure processingInstruction(var strTarget: WideString; var strData: WideString); safecall;
    procedure skippedEntity(var strName: WideString); safecall;
    property documentLocator: IVBSAXLocator write Set_documentLocator;

    procedure parse(varInput: OleVariant); safecall;
    procedure parseURL(const strURL: WideString); safecall;
    procedure flush;

    function getFeature(const strName: WideString): WordBool; safecall;
    procedure putFeature(const strName: WideString; fValue: WordBool); safecall;
    function getProperty(const strName: WideString): OleVariant; safecall;
    procedure putProperty(const strName: WideString; varValue: OleVariant); safecall;

    procedure WriteStartElement(strNamespaceURI, strLocalName, strQName: widestring; oAttributes: IVBSAXAttributes);
    procedure WriteEndElement(strNamespaceURI, strLocalName, strQName: WideString);
    procedure WriteAttributeDecl(strElementName, strAttributeName, strType, strValueDefault, strValue: WideString);
    procedure WriteStartDTD(strName, strPublicId, strSystemId: WideString);
    procedure WriteElementDecl(strName, strModel: WideString);
    procedure WriteCharacters(strChars: Widestring);
    procedure WriteComment(strChars: Widestring);

    property entityResolver: IVBSAXEntityResolver read Get_entityResolver write Set_entityResolver;
    property dtdHandler: IVBSAXDTDHandler read Get_dtdHandler write Set_dtdHandler;
    property errorHandler: IVBSAXErrorHandler read Get_errorHandler write Set_errorHandler;
    property contentHandler: IVBSAXContentHandler read Get_contentHandler write Set_contentHandler;

    property output: OleVariant read Get_output write Set_output;
    property parent: IVBSAXXMLReader read Get_parent write Set_parent;
    { Published declarations }

  published

//IMXWriter
    property writing: boolean read FWriting write FWriting;
    property encoding: WideString read Get_encoding write Set_encoding;
    property byteOrderMark: Boolean read Get_byteOrderMark write Set_byteOrderMark;
    property indent: WordBool read Get_indent write Set_indent;
    property standalone: WordBool read Get_standalone write Set_standalone;
    property omitXMLDeclaration: WordBool read Get_omitXMLDeclaration write Set_omitXMLDeclaration;
    property version: WideString read Get_version write Set_version;
    property disableOutputEscaping: WordBool read Get_disableOutputEscaping write Set_disableOutputEscaping;
//IVBReader
    property baseURL: WideString read GetbaseURL write SetbaseURL;
    property secureBaseURL: WideString read GetsecureBaseURL write SetsecureBaseURL;
// IVBSAXContentHandler
    property OnStartDocument: TstartDocumentEvent read FStartDocument write FStartDocument;
    property OnEndDocument: TEndDocumentEvent read FEndDocument write FEndDocument;
    property OnStartPrefixMapping: TstartPrefixMappingEvent read FStartPrefixMapping write FStartPrefixMapping;
    property OnEndPrefixMapping: TEndPrefixMappingEvent read FEndPrefixMapping write FEndPrefixMapping;
    property OnStartElement: TstartElementEvent read FStartElement write FStartElement;
    property OnEndElement: TEndElementEvent read FEndElement write FEndElement;
    property OnCharacters: TcharactersEvent read FCharacters write FCharacters;
    property OnIgnorableWhiteSpace: TignorableWhitespaceEvent read FIgnorableWhiteSpace write FIgnorableWhiteSpace;
    property OnProcessingInstruction: TprocessingInstructionEvent read FProcessingInstruction write FProcessingInstruction;
    property OnSkippedEntity: TskippedEntityEvent read FSkippedEntity write FSkippedEntity;
//IVBSaxDTDHandler
    property OnNotationDecl: TnotationDeclEvent read FNotationDecl write FNotationDecl;
    property OnUnparsedEntityDecl: TunparsedEntityDeclEvent read FUnparsedEntityDecl write FUnparsedEntityDecl;
// IVBSAXDeclHandler
    property OnElementDecl: TelementDeclEvent read FElementDecl write FElementDecl;
    property OnAttributeDecl: TattributeDeclEvent read FAttributeDecl write FAttributeDecl;
    property OnInternalEntityDecl: TinternalEntityDeclEvent read FInternalEntityDecl write FInternalEntityDecl;
    property OnExternalEntityDecl: TExternalEntityDeclEvent read FExternalEntityDecl write FExternalEntityDecl;
// IVBSAXLexicalHandler
    property OnStartDTD: TstartDTDEvent read FStartDTD write FStartDTD;
    property OnEndDTD: TendDTDEvent read FEndDTD write FEndDTD;
    property OnStartEntity: TstartEntityEvent read FStartEntity write FStartEntity;
    property OnEndEntity: TendEntityEvent read FEndEntity write FEndEntity;
    property OnStartCData: TstartCDATAEvent read FStartCData write FStartCData;
    property OnEndCData: TendCDATAEvent read FEndCData write FEndCData;
    property OnComment: TcommentEvent read FComment write FComment;
//IVBSaxErrorHandler
    property OnError: TerrorEvent read FError write FError;
    property OnFatalError: TFatalerrorEvent read FFatalError write FFatalError;
    property OnIgnorableWarning: TignorableWarningEvent read FIgnorableWarning write FIgnorableWarning;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TXMLParser]);
end;


{ TXMLAttributes }

procedure MakeSpace(var S: string);
begin
  if s = '' then S := ' ';
end;

procedure MakeWSpace(var S: widestring);
begin
  if s = '' then S := ' ';
end;

function TXMLAttributes.CreateAttribute(uri, localName, qName, aType, value: string): TAttribute;
var
  att: TAttribute;
begin
  att := TAttribute.Create;
  att.uri := uri;
  att.localName := localName;
  att.qName := qName;
  att.aType := aType;
  att.value := value;
  Result := Att;
end;

procedure TXMLAttributes.addAttribute(uri, localName, qName, aType, value: string);
begin
  MakeSpace(uri);
  MakeSpace(localname);
  MakeSpace(QName);
  MakeSpace(aType);
  MakeSpace(Value);
  attrList.Add(CreateAttribute(uri, localName, qName, aType, value));
end;

procedure TXMLAttributes.setAttribute(index: Integer; uri, localName, qName, aType, Value: string);
begin
  AttrList.Insert(index, createAttribute(uri, localName, qName, aType, value));
end;

procedure TXMLAttributes.SetAttributes(atts: IVBSAXAttributes);
var
  i: Integer;
begin
  for i := 0 to atts.length - 1 do
    addAttribute(atts.getURI(i), atts.getLocalName(i), atts.getQName(i), atts.getType(i), atts.getValue(i));
end;

procedure TXMLAttributes.removeAttribute(index: Integer);
begin
  attrList.Delete(index);
end;

function TXMLAttributes.findIndexByNSName(strURI, strLocalName: string): Integer;
var
  i: Integer;
begin
  for i := 0 to AttrList.Count - 1 do
  begin
    if (TAttribute(AttrList.Items[i]).uri = strURI) and (TAttribute(AttrList.Items[i]).localName = strLocalName)
      then begin
      result := i;
      Exit;
    end;
  end;
  Result := -1;
end;

function TXMLAttributes.findIndexByQName(strQName: string): Integer;
var
  i: Integer;
begin
  for i := 0 to AttrList.Count - 1 do
  begin
    if (TAttribute(AttrList.Items[i]).qName = strQName)
      then begin
      result := i;
      Exit;
    end;
  end;
  Result := -1;
end;

function TXMLAttributes.Get_length: SYSINT;
begin
  Result := AttrList.Count;
end;

function TXMLAttributes.getIndexFromName(const strURI,
  strLocalName: WideString): SYSINT;
begin
  result := findIndexByNSName(strURI, strLocalName);
end;

function TXMLAttributes.getIndexFromQName(
  const strQName: WideString): SYSINT;
begin
  result := findIndexByQName(strQName);
end;

function TXMLAttributes.getLocalName(nIndex: SYSINT): WideString;
begin
  Result := TAttribute(AttrList.Items[nIndex]).LocalName;
end;

function TXMLAttributes.getQName(nIndex: SYSINT): WideString;
begin
  Result := TAttribute(AttrList.Items[nIndex]).QName;
end;

function TXMLAttributes.getType(nIndex: SYSINT): WideString;
begin
  Result := TAttribute(AttrList.Items[nIndex]).aType;
end;

function TXMLAttributes.getTypeFromName(const strURI,
  strLocalName: WideString): WideString;
begin
  Result := TAttribute(AttrList.Items[FindIndexByNSName(strURI, strLocalName)]).aType;
end;

function TXMLAttributes.getTypeFromQName(
  const strQName: WideString): WideString;
begin
  Result := TAttribute(AttrList.Items[FindIndexByQName(strQName)]).aType;
end;

function TXMLAttributes.getURI(nIndex: SYSINT): WideString;
begin
  Result := TAttribute(AttrList.Items[nIndex]).uri;
end;

function TXMLAttributes.getValue(nIndex: SYSINT): WideString;
begin
  Result := TAttribute(AttrList.Items[nIndex]).value;
end;

function TXMLAttributes.getValueFromName(const strURI,
  strLocalName: WideString): WideString;
begin
  Result := TAttribute(AttrList.Items[FindIndexByNSName(strURI, strLocalName)]).value;
end;

function TXMLAttributes.getValueFromQName(
  const strQName: WideString): WideString;
begin
  Result := TAttribute(AttrList.Items[FindIndexByQName(strQName)]).value;
end;

constructor TXMLAttributes.Create;
begin
  inherited;
  AttrList := TObjectList.Create;
end;

destructor TXMLAttributes.destroy;
begin
  Attrlist.Free;
  inherited;
end;


function TXMLAttributes.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TXMLAttributes.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
  Result := E_NOTIMPL;
end;

function TXMLAttributes.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TXMLAttributes.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

procedure TXMLAttributes.clear;
begin
  AttrList.Clear;
end;

{ TXMLParser }


constructor TXMLParser.Create(Owner: TComponent);
begin
  inherited;
  Reader := CoSaxXMLReader30.Create;
  Writer := CoMXXMLWriter30.create;

  WContentHandler := Writer as IVBSaxContenthandler;
  WErrorHandler := Writer as IVBSaxErrorHandler;
  WDTDHandler := Writer as IVBSaxDTDHandler;
  WLexicalHandler := Writer as IVBSaxLexicalHandler;
  WDeclHandler := writer as IVBSaxDeclHandler;

  reader.ContentHandler := self;
  reader.ErrorHandler := self;
  reader.dtdHandler := self;
  reader.putProperty('http://xml.org/sax/properties/declaration-handler', IVBSaxDeclHandler(self));
  reader.putProperty('http://xml.org/sax/properties/lexical-handler', IVBSaxLexicalHandler(self));
end;


function TXMLParser.Get_baseURL: WideString;
begin
  result := Reader.Get_baseURL;
end;

function TXMLParser.Get_contentHandler: IVBSAXContentHandler;
begin
  Result := Reader.Get_contentHandler;
end;

function TXMLParser.Get_dtdHandler: IVBSAXDTDHandler;
begin
  Result := Reader.Get_dtdHandler;
end;

function TXMLParser.Get_entityResolver: IVBSAXEntityResolver;
begin
  result := Reader.Get_entityResolver
end;

function TXMLParser.Get_errorHandler: IVBSAXErrorHandler;
begin
  result := reader.Get_errorHandler
end;

function TXMLParser.getFeature(const strName: WideString): WordBool;
begin
  result := reader.getFeature(strName);
end;

function TXMLParser.getProperty(const strName: WideString): OleVariant;
begin
  result := reader.getProperty(strName);
end;


function TXMLParser.Get_secureBaseURL: WideString;
begin
  result := reader.Get_secureBaseURL;
end;

procedure TXMLParser.Set_parent(const oReader: IVBSAXXMLReader);
begin
  Reader := oReader;
end;

function TXMLParser.Get_parent: IVBSAXXMLReader;
begin
  result := Reader;
end;

procedure TXMLParser.parse(varInput: OleVariant);
begin
  Reader.parse(varInput);
end;

procedure TXMLParser.parseURL(const strURL: WideString);
begin
  Reader.parseURL(strUrl);
end;


procedure TXMLParser.putFeature(const strName: WideString;
  fValue: WordBool);
begin
  reader.putFeature(strName, fValue);
end;

procedure TXMLParser.putProperty(const strName: WideString;
  varValue: OleVariant);
begin
  reader.putProperty(strName, varValue);
end;

procedure TXMLParser.Set_baseURL(const strBaseURL: WideString);
begin
  reader.Set_baseURL(strBaseURL);
end;

procedure TXMLParser.Set_contentHandler(
  const oHandler: IVBSAXContentHandler);
begin
  FContentHandler := oHandler;
end;


procedure TXMLParser.Set_dtdHandler(const oHandler: IVBSAXDTDHandler);
begin
  Reader.Set_dtdHandler(oHandler);
end;

procedure TXMLParser.Set_entityResolver(
  const oResolver: IVBSAXEntityResolver);
begin
  reader.Set_entityResolver(oResolver);
end;

procedure TXMLParser.Set_errorHandler(const oHandler: IVBSAXErrorHandler);
begin
  reader.Set_errorHandler(oHandler);
end;


procedure TXMLParser.Set_secureBaseURL(const strSecureBaseURL: WideString);
begin
  reader.Set_secureBaseURL(strSecureBaseURL);
end;

procedure TXMLParser.characters(var strChars: WideString);
begin
  if Assigned(FCharacters) then FCharacters(strChars);
  if Assigned(FContentHandler) then FContenthandler.Characters(strChars);
  if Fwriting then WContenthandler.Characters(strChars);
end;

procedure TXMLParser.endDocument;
begin
  if Assigned(FendDocument) then FEndDocument;
  if assigned(FContenthandler) then FContentHandler.EndDocument;
  if FWriting then WContentHandler.EndDocument;
end;

procedure TXMLParser.endElement(var strNamespaceURI, strLocalName,
  strQName: WideString);
begin
  if Assigned(FEndElement) then FEndElement(strNamespaceURI, strLocalName, strQName);
  if assigned(FContenthandler) then
    FContentHandler.endelement(strNameSPaceURI, strLocalname, strQName);
  if FWriting then
    WContentHandler.endelement(strNameSPaceURI, strLocalname, strQName);
end;

procedure TXMLParser.endPrefixMapping(var strPrefix: WideString);
begin
  if Assigned(FEndprefixMapping) then FEndPrefixMapping(StrPrefix);
  if assigned(FContenthandler) then FContentHandler.endPrefixMapping(strPrefix);
  if FWriting then WContentHandler.endPrefixMapping(strPrefix);
end;

procedure TXMLParser.ignorableWhitespace(var strChars: WideString);
begin
  if Assigned(FignorableWhiteSpace) then FignorableWhiteSpace(StrChars);
  if assigned(FContenthandler) then FContentHandler.ignorableWhitespace(strchars);
  if FWriting then WContentHandler.ignorableWhitespace(strchars);
end;

procedure TXMLParser.processingInstruction(var strTarget,
  strData: WideString);
begin
  if Assigned(FprocessingInstruction) then FprocessingInstruction(strTarget, strData);
  if assigned(FContenthandler) then FContentHandler.processingInstruction(strTarget, strData);
  if FWriting then WContentHandler.processingInstruction(strTarget, strData);
end;

procedure TXMLParser.Set_documentLocator(const Param1: IVBSAXLocator);
begin
  Locator := param1;

end;

procedure TXMLParser.skippedEntity(var strName: WideString);
begin
  if Assigned(FSkippedEntity) then FSkippedEntity(strName);
  if assigned(FContenthandler) then FContentHandler.skippedEntity(strName);
  if FWriting then WContentHandler.skippedEntity(strName);
end;

procedure TXMLParser.startDocument;
begin
  if Assigned(FContenthandler) then FContentHandler.startDocument;
  if Assigned(FstartDocument) then FStartDocument;
  if fwriting then wContenthandler.startDocument;
end;

procedure TXMLParser.startElement(var strNamespaceURI, strLocalName,
  strQName: WideString; const oAttributes: IVBSAXAttributes);
begin
  if Assigned(FstartElement) then FStartElement(strNamespaceURI, strLocalName, strQName, oAttributes);
  if assigned(FContenthandler) then FContentHandler.StartElement(strNameSpaceURI, strLocalname, strQNAme, oAttributes);
  if FWriting then WContentHandler.StartElement(strNameSpaceURI, strLocalname, strQNAme, oAttributes);
end;

procedure TXMLParser.startPrefixMapping(var strPrefix, strURI: WideString);
begin
  if Assigned(FstartPrefixMapping) then FstartPrefixMapping(strPrefix, strURI);
  if assigned(FContenthandler) then FContentHandler.StartPrefixMapping(strPrefix, strURI);
  if FWriting then WContentHandler.StartPrefixMapping(strPrefix, strURI);
end;

procedure TXMLParser.error(const oLocator: IVBSAXLocator;
  var strErrorMessage: WideString; nErrorCode: Integer);
begin
  if Assigned(FError) then FError(oLocator, strErrorMessage, nErrorCode);
  if Assigned(FErrorHandler) then FErrorHandler.error(olocator, strErrorMessage, nErrorCode);
  if FWriting then WErrorHandler.error(olocator, strErrorMessage, nErrorCode);
end;

procedure TXMLParser.fatalError(const oLocator: IVBSAXLocator;
  var strErrorMessage: WideString; nErrorCode: Integer);
begin
  if Assigned(FfatalError) then FfatalError(oLocator, strErrorMessage, nErrorCode);
  if Assigned(FErrorHandler) then FErrorHandler.fatalError(olocator, strErrorMessage, nErrorCode);
  if FWriting then WErrorHandler.fatalError(olocator, strErrorMessage, nErrorCode);
end;

procedure TXMLParser.ignorableWarning(const oLocator: IVBSAXLocator;
  var strErrorMessage: WideString; nErrorCode: Integer);
begin
  if Assigned(FignorableWarning) then FignorableWarning(oLocator, strErrorMessage, nErrorCode);
  if Assigned(FErrorHandler) then FErrorHandler.ignorablewarning(olocator, strErrorMessage, nErrorCode);
  if FWriting then WErrorHandler.ignorablewarning(olocator, strErrorMessage, nErrorCode);
end;

procedure TXMLParser.notationDecl(var strName, strPublicId,
  strSystemId: WideString);
begin
  if Assigned(FnotationDecl) then FnotationDecl(strName, strPublicID, strSystemID);
  if Assigned(FDTDHandler) then FDTDHandler.notationDecl(StrName, strPublicID, strSystemID);
  if FWriting then WDTDHandler.notationDecl(StrName, strPublicID, strSystemID);
end;

procedure TXMLParser.unparsedEntityDecl(var strName, strPublicId,
  strSystemId, strNotationName: WideString);
begin
  if Assigned(FunparsedEntityDecl) then FunparsedEntityDecl(strName, strPublicID, strSystemID, strNotationName);
  if Assigned(FDTDHandler) then FDTDHandler.unparsedEntityDecl(strname, strPublicID, strSystemID, strNotationName);
  if FWriting then WDTDHandler.unparsedEntityDecl(strname, strPublicID, strSystemID, strNotationName);
end;

procedure TXMLParser.comment(var strChars: WideString);
begin
  if Assigned(Fcomment) then Fcomment(strChars);
  if Assigned(FLexicalHandler) then FLexicalHandler.comment(strChars);
  if FWriting then WLexicalHandler.comment(strChars);
end;

procedure TXMLParser.endCDATA;
begin
  if Assigned(FendCDATA) then FendCDATA();
  if FWriting then WLexicalHandler.endCData;
end;

procedure TXMLParser.endDTD;
begin
  if Assigned(FendDTD) then FendDTD();
  if Assigned(FLexicalHandler) then FLexicalHandler.endDTD;
  if FWriting then WLexicalHandler.endDTD;
end;

procedure TXMLParser.endEntity(var strName: WideString);
begin
  if Assigned(FendEntity) then FendEntity(strName);
  if Assigned(FLexicalHandler) then FLexicalHandler.EndEntity(strname);
  if FWriting then WLexicalHandler.EndEntity(strname);
end;

procedure TXMLParser.startCDATA;
begin
  if Assigned(FstartCDATA) then FstartCDATA();
  if Assigned(FLexicalHandler) then FLexicalHandler.startCDATA;
  if FWriting then WLexicalHandler.startCDATA;
end;

procedure TXMLParser.startDTD(var strName, strPublicId,
  strSystemId: WideString);
begin
  if Assigned(FstartDTD) then FstartDTD(strName, strPublicID, strSystemID);
  if Assigned(FLexicalHandler) then FLexicalHandler.StartDTD(strname, strPublicID, strSystemID);
  if FWriting then WLexicalHandler.StartDTD(strname, strPublicID, strSystemID);
end;

procedure TXMLParser.startEntity(var strName: WideString);
begin
  if Assigned(FstartEntity) then FstartEntity(strName);
  if Assigned(FLexicalHandler) then FLexicalHandler.StartEntity(strName);
  if FWriting then WLexicalHandler.StartEntity(strName);
end;

procedure TXMLParser.attributeDecl(var strElementName, strAttributeName,
  strType, strValueDefault, strValue: WideString);
begin
  if Assigned(FattributeDecl) then FattributeDecl(strElementName,
      strAttributeName, strType, strValueDefault, strValue);
  if Assigned(FDeclHandler) then FDeclHandler.attributeDecl(strElementName, strAttributeName, strType, StrValueDefault, strValue);
  if FWriting then WDeclHandler.attributeDecl(strElementName, strAttributeName, strType, StrValueDefault, strValue);
end;

procedure TXMLParser.elementDecl(var strName, strModel: WideString);
begin
  if Assigned(FelementDecl) then FelementDecl(strName, strModel);
  if Assigned(FDeclHandler) then FDeclHandler.elementDecl(StrName, strModel);
  if FWriting then WDeclHandler.elementDecl(StrName, strModel);
end;

procedure TXMLParser.externalEntityDecl(var strName, strPublicId,
  strSystemId: WideString);
begin
  if Assigned(FexternalEntityDecl) then FexternalEntityDecl(strName, strPublicID, strSystemID);
  if Assigned(FDeclHandler) then FDeclHandler.externalEntityDecl(strName, strPublicID, strSystemID);
  if FWriting then WDeclHandler.externalEntityDecl(strName, strPublicID, strSystemID);
end;

procedure TXMLParser.internalEntityDecl(var strName, strValue: WideString);
begin
  if Assigned(FinternalEntityDecl) then FinternalEntityDecl(strName, StrValue);
  if Assigned(FDeclHandler) then FDeclHandler.internalEntityDecl(strName, StrValue);
  if FWriting then WDeclHandler.internalEntityDecl(strName, StrValue);
end;


function TXMLParser.GetbaseURL: WideString;
begin
  result := reader.Get_BaseURL;
end;

function TXMLParser.GetsecureBaseURL: WideString;
begin
  result := reader.Get_secureBaseURL;
end;

procedure TXMLParser.SetbaseURL(const strBaseURL: WideString);
begin
  reader.Set_secureBaseURL(StrBaseURL);
end;

procedure TXMLParser.SetsecureBaseURL(const strSecureBaseURL: WideString);
begin
  reader.Set_secureBaseURL(StrSecureBaseURL);
end;

procedure TXMLParser.flush;
begin
  writer.flush;
end;

function TXMLParser.Get_byteOrderMark: Boolean;
begin
  Result := Writer.Get_byteOrderMark;
end;

procedure TXMLParser.Set_byteOrderMark(fWriteByteOrderMark: Boolean);
begin
  Writer.Set_byteOrderMark(fWriteByteOrderMark);
end;


function TXMLParser.Get_disableOutputEscaping: WordBool;
begin
  Result := Writer.Get_disableOutputEscaping;
end;

function TXMLParser.Get_encoding: WideString;
begin
  Result := Writer.Get_encoding;
end;

function TXMLParser.Get_indent: WordBool;
begin
  Result := Writer.Get_indent;
end;

function TXMLParser.Get_omitXMLDeclaration: WordBool;
begin
  Result := Writer.Get_omitXMLDeclaration;
end;

function TXMLParser.Get_output: OleVariant;
begin
  Result := Writer.Get_output;
end;

function TXMLParser.Get_standalone: WordBool;
begin
  Result := Writer.Get_standalone;
end;

function TXMLParser.Get_version: WideString;
begin
  Result := Writer.Get_version;
end;


procedure TXMLParser.Set_disableOutputEscaping(fValue: WordBool);
begin
  Writer.Set_disableOutputEscaping(Boolean(fValue));
end;

procedure TXMLParser.Set_encoding(const strEncoding: WideString);
begin
  Writer.Set_encoding(strEncoding);
end;

procedure TXMLParser.Set_indent(fIndentMode: WordBool);
begin
  Writer.Set_indent(Boolean(fIndentMode));
end;

procedure TXMLParser.Set_omitXMLDeclaration(fValue: WordBool);
begin
  Writer.Set_omitXMLDeclaration(Boolean(fValue));
end;

procedure TXMLParser.Set_output(varDestination: OleVariant);
begin
  Writer.Set_output(varDestination);
end;

procedure TXMLParser.Set_standalone(fValue: WordBool);
begin
  Writer.Set_standalone(Boolean(fValue));
end;

procedure TXMLParser.Set_version(const strVersion: WideString);
begin
  Writer.Set_version(strVersion);
end;


procedure TXMLParser.WriteStartElement(strNamespaceURI, strLocalName,
  strQName: wideString; oAttributes: IVBSAXAttributes);
begin
  MakeWSpace(StrNameSpaceURI);
  MakeWSpace(StrQName);
  MakeWSpace(StrLocalName);
  Startelement(StrNameSpaceURI, strLocalName, strQName, oAttributes);
end;

procedure TXMLParser.WriteAttributeDecl(strElementName, strAttributeName,
  strType, strValueDefault, strValue: WideString);
begin
  AttributeDecl(strElementName, strAttributeName,
    strType, strValueDefault, strValue);
end;

procedure TXMLParser.WriteCharacters(strChars: Widestring);
begin
  Characters(strChars);
end;

procedure TXMLParser.WriteElementDecl(strName, strModel: WideString);
begin
  ElementDecl(strname, strModel);
end;

procedure TXMLParser.WriteEndElement(strNamespaceURI, strLocalName,
  strQName: WideString);
begin
  MakeWSpace(StrNameSpaceURI);
  MakeWSpace(StrQName);
  MakeWSpace(StrLocalName);
  EndElement(strNameSpaceURI, strLocalName, strQName);
end;

procedure TXMLParser.WriteStartDTD(strName, strPublicId,
  strSystemId: WideString);
begin
  StartDTD(strName, strPublicID, strSystemID);
end;

procedure TXMLParser.WriteComment(strChars: Widestring);
begin
  comment(strChars);
end;

initialization

  coinitialize(nil);

finalization

  couninitialize;

end.

